/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * MapRegion.h
 *
 *  Created on: Oct 13, 2010
 *      Author: ltwardon
 */

#ifndef MAPREGION_H_
#define MAPREGION_H_

#include <string>
#include <list>
#include "MapPoint.h"
#include "../navigation/PositionData.h"
#include "../navigation/NavigationGoalData.h"
#include "../../units/UnitConverter.h"

namespace airobots {

class MapRegion {

public:

	static const std::string POLYGONE_TYPE;
	static const std::string Rectangle_TYPE;

	MapRegion();
	MapRegion(std::string type);
	MapRegion(std::string type, std::string label);
	MapRegion(std::string type, std::string label, std::string group);
	MapRegion(std::string type, std::string label, std::string group,
			std::list<MapPoint> points);
	MapRegion(std::string type, std::string label, std::string group,
			std::list<MapPoint> points, NavigationGoalData viewPoint);
	MapRegion(std::string type, std::string label, std::string group,
			std::list<MapPoint> points, NavigationGoalData viewPoint, double height,LengthUnit lU);

	/*
	 * @param lengthUnit: unit for scale of x and y
	 * */
	void addPoint(double x, double y,LengthUnit lengthUnit);
	void addPoint(MapPoint point);
	std::string getType() const;
	void setType(std::string type);
	std::string getLabel() const;
	void setLabel(std::string label);
	std::string getGroup() const;
	void setGroup(std::string group);
	std::list<MapPoint> getPoints() const;
	void setPoints(std::list<MapPoint> points);
	NavigationGoalData getViewPoint() const;
	void setViewPoint(NavigationGoalData viewPoint);
	double getHeight(LengthUnit lU) const ;
	void setHeight(double height,LengthUnit lU);
	void removePoint(double x, double y);
	int getPointCount() const;
	PositionData getPositionData() const;
	bool isViewPointSet() const;
	bool contains(double x, double y) const;

private:

	std::string type;
	std::string label;
	std::string group;
	std::list<MapPoint> points;
	NavigationGoalData viewPoint;
	double height;
	bool viewPointSet;
	static const LengthUnit iLU = METER;

};

}

#endif /* MAPREGION_H_ */

