/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * MicroTimestamp.h
 *
 *  Created on: Aug 6, 2009
 *      Author: jwienke
 */

#ifndef MICROTIMESTAMP_H_
#define MICROTIMESTAMP_H_

#include <string>
#include <sstream>

#include <sys/time.h>

#include <xmltio/xmltio.hpp>

namespace airobots {

/**
 * A timestamp with microseconds precision.
 */
class MicroTimestamp {
public:

	static timeval timevalFromDouble(double time);

	static std::string getBaseTag();

	static std::string getTemplate();

	MicroTimestamp();
	MicroTimestamp(timeval readtime);
	MicroTimestamp(double time);
	MicroTimestamp(time_t seconds, suseconds_t microSeconds);

	timeval getTimeval() const;
	time_t getSeconds() const;
	suseconds_t getMicroSeconds() const;

	void set(timeval time);

	bool operator==(const MicroTimestamp & other) const;

private:
	timeval readtime;

};

std::ostream& operator<<(std::ostream& o, MicroTimestamp const& timestamp);

}

namespace xmltio {

void convert(const airobots::MicroTimestamp& ld, Location& loc);

template<>
airobots::MicroTimestamp xmltio::extract<airobots::MicroTimestamp>::get() const;

}

#endif /* MICROTIMESTAMP_H_ */
