/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * KatanaPoseData.h
 *
 *  Created on: May 25, 2010
 *      Author: ttoenige
 */

#ifndef KATANAPOSEDATA_H_
#define KATANAPOSEDATA_H_

#include "../../Type.h"
#include "../../units/UnitConverter.h"
namespace airobots
{

class KatanaPoseData: public Type
{
public:
	static std::string getBaseTag();

	static std::string getTemplate();

	/**
	 * Returns the x-value of the endeffector position (mm)
	 */
	double getX(LengthUnit lengthUnit) const;
	void setX(double x,LengthUnit lengthUnit);

	/**
	 * Returns the y-value of the endeffector position (mm)
	 */
	double getY(LengthUnit lengthUnit) const;
	void setY(double y,LengthUnit lengthUnit);

	/**
	 * Returns the z-value of the endeffector position (mm)
	 */
	double getZ(LengthUnit lengthUnit) const;
	void setZ(double z,LengthUnit lengthUnit);

	/**
	 * Returns the phi-value of the endeffector position (rad)
	 */
	double getPhi(AngleUnit angleUnit) const;
	void setPhi(double phi,AngleUnit angleUnit);

	/**
	 * Returns the psi-value of the endeffector position (rad)
	 */
	double getPsi(AngleUnit angleUnit) const;
	void setPsi(double psi,AngleUnit angleUnit);

	/**
	 * Returns the theta-value of the endeffector position (rad)
	 */
	double getTheta(AngleUnit angleUnit) const;
	void setTheta(double theta,AngleUnit angleUnit);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, KatanaPoseData &type);

	KatanaPoseData();

private:
	double x;
	double y;
	double z;
	double theta;
	double phi;
	double psi;
};

}

#endif /* KATANAPOSEDATA_H_ */
