/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * CameraTargetData.cpp
 *
 *  Created on: Aug 6, 2009
 *      Author: jwienke
 */

#include "CameraTargetData.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

std::string CameraTargetData::getBaseTag() {
	return "CAMERATARGET";
}

std::string CameraTargetData::getTemplate() {
	return std::string("<") + getBaseTag() + "><GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<CYLINDRIC dist=\"\" height=\"\"/>"
			+ "<AZIMUTH rad=\"\" />"
			+ "<ZOOM qual=\"\" />" + "</" + getBaseTag() + ">";
}

CameraTargetData::CameraTargetData() :
	Type("unknown") {
}

double CameraTargetData::getDistance(LengthUnit lengthUnit) const {
	return UnitConverter::convert(distance,MILLIMETER,lengthUnit);
}

void CameraTargetData::setDistance(double distance,LengthUnit lengthUnit) {
	this->distance = UnitConverter::convert(distance,lengthUnit,MILLIMETER);
}

double CameraTargetData::getHeight(LengthUnit lengthUnit) const {
	return UnitConverter::convert(height,MILLIMETER,lengthUnit);
}

void CameraTargetData::setHeight(double height, LengthUnit lengthUnit) {
	this->height = UnitConverter::convert(height,lengthUnit,MILLIMETER);;
}

double CameraTargetData::getAzimuth(AngleUnit angleUnit) const {
	return UnitConverter::convert(azimuth,RADIAN,angleUnit);
}

void CameraTargetData::setAzimuth(double azimuth,AngleUnit angleUnit) {
	this->azimuth = UnitConverter::convert(azimuth,angleUnit,RADIAN);
}

double CameraTargetData::getZoom() const {
	return zoom;
}

void CameraTargetData::setZoom(double zoom) {
	this->zoom = zoom;
}

void CameraTargetData::sanitizeLocation(xmltio::Location &loc) const {
	
}

void CameraTargetData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("CYLINDRIC/@dist")] = getDistance(MILLIMETER);
	loc[XPath("CYLINDRIC/@height")] = getHeight(MILLIMETER);

	loc[XPath("AZIMUTH/@rad")] = getAzimuth(RADIAN);

	loc[XPath("ZOOM/@qual")] = getZoom();

}

void CameraTargetData::fromLocation(const xmltio::Location &loc_, CameraTargetData &ctd) {

	Type::fromLocation(loc_, ctd);

	ctd.setDistance(extract<double> (loc_[XPath("CYLINDRIC/@dist")]),MILLIMETER);
	ctd.setHeight(extract<double> (loc_[XPath("CYLINDRIC/@height")]),MILLIMETER);

	ctd.setAzimuth(extract<double> (loc_[XPath("AZIMUTH/@rad")]),RADIAN);

	ctd.setZoom(extract<double> (loc_[XPath("ZOOM/@qual")]));

}
