/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */
 

/* 
 * File:   Viewpoint.cpp
 * Author: lkettenb
 * 
 * Created on April 17, 2012, 12:34 PM
 */

#include "Viewpoint.h"

airobots::Viewpoint::Viewpoint(airobots::PositionData _coordinates,
        ViewpointCategory _category,
        std::string _label)
:
coordinates(_coordinates),
category(_category),
label(_label) {

}

std::string airobots::Viewpoint::getBaseTag() {
    return "VIEWPOINT";
}

std::string airobots::Viewpoint::getTemplate() {
    return "<" + getBaseTag() + " label=\"\" category=\"\" >" +
            airobots::Timestamp::getTemplate() + 
            "<GENERATOR/>" +
            "</" + getBaseTag() + ">";
}

void airobots::Viewpoint::fillInto(xmltio::Location& parent) const {
    airobots::Type::fillInto(parent);
    
    // Set attributes.
    parent[xmltio::XPath("@label")] = label;
    switch (category) {
        case VIEW: parent[xmltio::XPath("@category")] = "VIEW"; break;
        case PERSON: parent[xmltio::XPath("@category")] = "PERSON"; break;
        case OBJECT: parent[xmltio::XPath("@category")] = "OBJECT"; break;
    }
    
    
    
    // Set coordinates.
    xmltio::Location item(airobots::PositionData::getTemplate(),
            airobots::PositionData::getBaseTag());
    item = coordinates;
    parent.add(item);
}

void airobots::Viewpoint::fromLocation(const xmltio::Location& location,
        Viewpoint& viewpoint) {
    airobots::Type::fromLocation(location, viewpoint);
    
    // Get attributes.
    if (xmltio::extract<std::string>(location[xmltio::XPath("@category")]).get() == "VIEW") {
        viewpoint.setCategory(VIEW);
    } else if (xmltio::extract<std::string>(location[xmltio::XPath("@category")]).get() == "OBJECT") {
        viewpoint.setCategory(OBJECT);
    } else if (xmltio::extract<std::string>(location[xmltio::XPath("@category")]).get() == "PERSON") {
        viewpoint.setCategory(PERSON);
    }
    
    viewpoint.setLabel(xmltio::extract<std::string>(location[xmltio::XPath("@label")]).get());
    
    // Get coordinates.
	for (xmltio::XPathIterator i = 
            xmltio::XPath(airobots::PositionData::getBaseTag()).evaluate(location); 
            i; 
            ++i) {
		xmltio::Location positionDataLocation(*i);
        airobots::PositionData p = 
                xmltio::extract<airobots::PositionData>(positionDataLocation);
        viewpoint.setCoordinates(p);
	}
}

void airobots::Viewpoint::sanitizeLocation(xmltio::Location& location) const {
    xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(location)) {
		xmltio::Location l(*i);
		location.remove(l);
	}
}

