/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */

#include "DriveData.h"
#include "../../units/UnitConverter.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

SpeedUnit DriveData::iSU = METER_PER_SEC;
LengthUnit DriveData::iLU = METER;

std::string DriveData::getBaseTag() {
	return "DRIVE";
}

std::string DriveData::getTemplate() {
	return std::string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + MicroTimestamp::getTemplate()
			+ "<SPEED quant=\"\"/>" + "<DISTANCE quant=\"\"/>"
			+ "</" + getBaseTag() + ">";
}

DriveData::DriveData() :
		Type("unknown") {
}

double DriveData::getSpeed(SpeedUnit u) const {
	return UnitConverter::convert(speed, iSU, u);
}

void DriveData::setSpeed(double speed, SpeedUnit u) {
	this->speed = UnitConverter::convert(speed, u, iSU);
}

double DriveData::getDistance(LengthUnit u) const {
	return UnitConverter::convert(distance, iLU, u);
}

void DriveData::setDistance(double distance, LengthUnit u) {
	this->distance = UnitConverter::convert(distance, u, iLU);
}

void DriveData::sanitizeLocation(xmltio::Location &loc) const {

}

void DriveData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();
	loc[XPath("DISTANCE/@quant")] = getDistance(iLU);
	loc[XPath("SPEED/@quant")] = getSpeed(iSU);

}

void DriveData::fromLocation(const xmltio::Location &loc_, DriveData &sd) {

	Type::fromLocation(loc_, sd);

	sd.setDistance(extract<double>(loc_[XPath("DISTANCE/@quant")]), iLU);
	sd.setSpeed(extract<double>(loc_[XPath("SPEED/@quant")]),
			iSU);

	sd.setReadTime(
			extract<MicroTimestamp>(loc_[XPath(MicroTimestamp::getBaseTag())]));

}
