/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */

#include <sstream>
#include <stdlib.h>

#include "GoalReachedData.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

std::string GoalReachedData::getBaseTag() {
	return "GOALREACHED";
}

std::string GoalReachedData::getTemplate() {
	return std::string("<" + getBaseTag() + ">") + "<GENERATOR/>"
			+ Timestamp::getTemplate() + "<REACHED/>" + "</" + getBaseTag()
			+ ">";
}

GoalReachedData::GoalReachedData() {
}

GoalReachedData::GoalReachedData(string generator, bool goalReached) :
	Type(generator), goalReached(goalReached) {
}

void GoalReachedData::setGoalReached(bool reached) {
	this->goalReached = reached;
}

bool GoalReachedData::isGoalReached() const {
	return goalReached;
}

void GoalReachedData::sanitizeLocation(xmltio::Location &loc) const {
	
}

void GoalReachedData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("REACHED")] = isGoalReached() ? 1 : 0;

}

void GoalReachedData::fromLocation(const xmltio::Location &loc_, GoalReachedData &grd) {

	Type::fromLocation(loc_, grd);

	grd.setGoalReached(extract<int> (loc_[XPath("REACHED")]));

}

