/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * PositionData.cpp
 *
 *  Created on: Jul 31, 2009
 *      Author: jwienke
 */

#include "PositionData.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

namespace airobots {

std::string PositionData::getBaseTag() {
	return "ROBOTPOSITION";
}

std::string PositionData::getTemplate() {

	return "<" + getBaseTag() + ">"
		+ "<GENERATOR/>"
		+ airobots::Timestamp::getTemplate()
		+ "<POSITION ref=\"world\" kind=\"absolute\" x=\"\" y=\"\" theta=\"\" />"
		+ "</" + getBaseTag() + ">";

}

PositionData::PositionData() :
	x(0), y(0), theta(0) {

}

PositionData::PositionData(float x, float y, float theta,LengthUnit lengthUnit,AngleUnit angleUnit) :
	x(UnitConverter::convert(x,lengthUnit,METER)), y(UnitConverter::convert(y,lengthUnit,METER)), theta(UnitConverter::convert(theta,angleUnit,RADIAN)) {

}

PositionData::~PositionData() {
}

float PositionData::getX(LengthUnit lengthUnit) const {
	return UnitConverter::convert(x,METER,lengthUnit);
}

void PositionData::setX(float x,LengthUnit lengthUnit) {
	this->x = UnitConverter::convert(x,lengthUnit,METER);
}

float PositionData::getY(LengthUnit lengthUnit) const {
	return UnitConverter::convert(y,METER,lengthUnit);
}

void PositionData::setY(float y,LengthUnit lengthUnit) {
	this->y = UnitConverter::convert(y,lengthUnit,METER);
}

float PositionData::getTheta(AngleUnit angleUnit) const {
	return UnitConverter::convert(theta,RADIAN,angleUnit);
}

void PositionData::setTheta(float theta,AngleUnit angleUnit) {
	this->theta = UnitConverter::convert(theta,angleUnit,RADIAN);
}

void PositionData::sanitizeLocation(xmltio::Location &loc) const {

}

void PositionData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("POSITION/@x")] = getX(METER);
	loc[XPath("POSITION/@y")] = getY(METER);
	loc[XPath("POSITION/@theta")] = getTheta(RADIAN);

}

void PositionData::fromLocation(const xmltio::Location &loc_, PositionData &pos) {

	Type::fromLocation(loc_, pos);

	pos.setX(extract<float> (loc_[XPath("POSITION/@x")]),METER);
	pos.setY(extract<float> (loc_[XPath("POSITION/@y")]),METER);
	pos.setTheta(extract<float> (loc_[XPath("POSITION/@theta")]),RADIAN);

}

}
