/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * ROSBaseLocalPlannerConfiguration.cpp
 *
 *  Created on: Jun 13, 2012
 *      Author: pdressel
 */

#include "ROSBaseLocalPlannerConfiguration.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

namespace airobots 
{
    const string ROSBaseLocalPlannerConfiguration::BASELOCALPLANNER_CONFIGURATION_TAG = "BASELOCALPLANNERCONFIGURATION";
    
    const string ROSBaseLocalPlannerConfiguration::ACC_LIM_X_ATTRIBUTE = "acc_lim_x";    
    const string ROSBaseLocalPlannerConfiguration::ACC_LIM_Y_ATTRIBUTE = "acc_lim_y";
    const string ROSBaseLocalPlannerConfiguration::ACC_LIM_THETA_ATTRIBUTE = "acc_lim_theta";
    const string ROSBaseLocalPlannerConfiguration::MAX_VEL_X_ATTRIBUTE = "max_vel_x";
    const string ROSBaseLocalPlannerConfiguration::MIN_VEL_X_ATTRIBUTE = "min_vel_x";
    const string ROSBaseLocalPlannerConfiguration::MAX_VEL_THETA_ATTRIBUTE = "max_vel_theta";
    const string ROSBaseLocalPlannerConfiguration::MIN_VEL_THETA_ATTRIBUTE = "min_vel_theta";
    const string ROSBaseLocalPlannerConfiguration::MIN_IN_PLACE_VEL_THETA_ATTRIBUTE = "min_in_place_vel_theta";
    const string ROSBaseLocalPlannerConfiguration::SIM_TIME_ATTRIBUTE = "sim_time";
    const string ROSBaseLocalPlannerConfiguration::SIM_GRANULARITY_ATTRIBUTE = "sim_granularity";
    const string ROSBaseLocalPlannerConfiguration::PDIST_SCALE_ATTRIBUTE = "pdist_scale";
    const string ROSBaseLocalPlannerConfiguration::GDIST_SCALE_ATTRIBUTE = "gdist_scale";
    const string ROSBaseLocalPlannerConfiguration::OCCDIST_SCALE_ATTRIBUTE = "occdist_scale";
    const string ROSBaseLocalPlannerConfiguration::OSCILLATION_RESET_DIST_ATTRIBUTE = "oscillation_reset_dist";
    const string ROSBaseLocalPlannerConfiguration::ESCAPE_RESET_DIST_ATTRIBUTE = "escape_reset_dist";
    const string ROSBaseLocalPlannerConfiguration::ESCAPE_RESET_THETA_ATTRIBUTE = "escape_reset_theta";
    const string ROSBaseLocalPlannerConfiguration::VX_SAMPLES_ATTRIBUTE = "vx_samples";
    const string ROSBaseLocalPlannerConfiguration::VTHETA_SAMPLES_ATTRIBUTE = "vtheta_samples";
    const string ROSBaseLocalPlannerConfiguration::HEADING_LOOKAHEAD_ATTRIBUTE = "heading_lookahead";
    const string ROSBaseLocalPlannerConfiguration::HOLONOMIC_ROBOT_ATTRIBUTE = "holonomic_robot";
    const string ROSBaseLocalPlannerConfiguration::ESCAPE_VEL_ATTRIBUTE = "escape_vel";
    const string ROSBaseLocalPlannerConfiguration::DWA_ATTRIBUTE = "dwa";
    const string ROSBaseLocalPlannerConfiguration::HEADING_SCORING_ATTRIBUTE = "heading_scoring";
    const string ROSBaseLocalPlannerConfiguration::HEADING_SCORING_TIMESTAMP_ATTRIBUTE = "heading_scoring_timestep";
    const string ROSBaseLocalPlannerConfiguration::SIMPLE_ATTRACTOR_ATTRIBUTE = "simple_attractor";
    const string ROSBaseLocalPlannerConfiguration::ANGULAR_SIM_GRANULARITY_ATTRIBUTE = "angular_sim_granularity";
    const string ROSBaseLocalPlannerConfiguration::Y_VELS_ATTRIBUTE = "y_vels";
    const string ROSBaseLocalPlannerConfiguration::RESTORE_DEFAULTS_ATTRIBUTE = "restore_defaults";

    ROSBaseLocalPlannerConfiguration::ROSBaseLocalPlannerConfiguration()
    {
        this->acc_lim_theta = 0.0;
        this->acc_lim_x = 0.0;
        this->acc_lim_y = 0.0;
        this->angular_sim_granularity = 0.0;
        this->dwa = false;
        this->escape_reset_dist = 0.0;
        this->escape_reset_theta = 0.0;
        this->escape_vel = 0.0;
        this->gdist_scale = 0.0;
        this->heading_lookahead = 0.0;
        this->heading_scoring = false;
        this->heading_scoring_timestep = 0.0;
        this->holonomic_robot = false;
        this->max_vel_theta = 0.0;
        this->max_vel_x = 0.0;
        this->min_in_place_vel_theta = 0.0;
        this->min_vel_theta = 0.0;
        this->min_vel_x = 0.0;
        this->occdist_scale = 0.0;
        this->oscillation_reset_dist = 0.0;
        this->pdist_scale = 0.0;
        this->restore_defaults = false;
        this->sim_granularity = 0.0;
        this->sim_time = 0.0;
        this->simple_attractor = false;
        this->vtheta_samples = 0;
        this->vx_samples = 0;      
        this->y_vels = "";
    }    
    
    string ROSBaseLocalPlannerConfiguration::getBaseTag()
    {
        return ROSDynamicConfiguration::getBaseTag();
    }
    
    string ROSBaseLocalPlannerConfiguration::getTemplate()
    {
    	return "<" + getBaseTag() + ">"		
		+ airobots::Timestamp::getTemplate()
                + "<GENERATOR/>"
		+ "<" + BASELOCALPLANNER_CONFIGURATION_TAG + " " + 
                        + "acc_lim_theta=\"\" "
                        + "acc_lim_x=\"\" " 
                        + "acc_lim_y=\"\" "
                        + "angular_sim_granularity=\"\" " 
                        + "dwa=\"\" "
                        + "escape_reset_dist=\"\" " 
                        + "escape_reset_theta=\"\" "
                        + "escape_vel=\"\" " 
                        + "gdist_scale=\"\" "
                        + "heading_lookahead=\"\" " 
                        + "heading_scoring=\"\" "
                        + "heading_scoring_timestep=\"\" " 
                        + "holonomic_robot=\"\" "                
                        + "max_vel_theta=\"\" " 
                        + "max_vel_x=\"\" "
                        + "min_in_place_vel_theta=\"\" " 
                        + "min_vel_theta=\"\" "
                        + "min_vel_x=\"\" " 
                        + "occdist_scale=\"\" "    
                        + "oscillation_reset_dist=\"\" "
                        + "pdist_scale=\"\" "  
			+ "restore_defaults=\"\" "
                        + "sim_granularity=\"\" "
                        + "sim_time=\"\" "  
                        + "simple_attractor=\"\" "
                        + "vtheta_samples=\"\" "  
                        + "vx_samples=\"\" "
                        + "y_vels=\"\"/>"  
		+ "</" + getBaseTag() + ">";
    }    
    
    void ROSBaseLocalPlannerConfiguration::sanitizeLocation(xmltio::Location &loc) const
    {
        ROSDynamicConfiguration::sanitizeLocation(loc);
    }
    
    void ROSBaseLocalPlannerConfiguration::fillInto(xmltio::Location &loc) const
    {
        string prefix = BASELOCALPLANNER_CONFIGURATION_TAG 
                + ROSDynamicConfiguration::X_PATH_SEPARATOR 
                + ROSDynamicConfiguration::ATTRIBUTE_PREFIX;
	
        Type::fillInto(loc);

        loc[XPath(prefix + ACC_LIM_THETA_ATTRIBUTE)]            = this->acc_lim_theta;
	loc[XPath(prefix + ACC_LIM_X_ATTRIBUTE)]                = this->acc_lim_x;
        loc[XPath(prefix + ACC_LIM_Y_ATTRIBUTE)]                = this->acc_lim_y;
        loc[XPath(prefix + ANGULAR_SIM_GRANULARITY_ATTRIBUTE)]  = this->angular_sim_granularity;
        loc[XPath(prefix + DWA_ATTRIBUTE)]                      = this->dwa == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + ESCAPE_RESET_DIST_ATTRIBUTE)]        = this->escape_reset_dist;
        loc[XPath(prefix + ESCAPE_RESET_THETA_ATTRIBUTE)]       = this->escape_reset_theta;
        loc[XPath(prefix + ESCAPE_VEL_ATTRIBUTE)]               = this->escape_vel;
        loc[XPath(prefix + GDIST_SCALE_ATTRIBUTE)]              = this->gdist_scale;
        loc[XPath(prefix + HEADING_LOOKAHEAD_ATTRIBUTE)]        = this->heading_lookahead;
        loc[XPath(prefix + HEADING_SCORING_ATTRIBUTE)]          = this->heading_scoring == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + HEADING_SCORING_TIMESTAMP_ATTRIBUTE)]= this->heading_scoring_timestep;
        loc[XPath(prefix + HOLONOMIC_ROBOT_ATTRIBUTE)]          = this->holonomic_robot == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + MAX_VEL_THETA_ATTRIBUTE)]            = this->max_vel_theta;
        loc[XPath(prefix + MAX_VEL_X_ATTRIBUTE)]                = this->max_vel_x;
        loc[XPath(prefix + MIN_IN_PLACE_VEL_THETA_ATTRIBUTE)]   = this->min_in_place_vel_theta;
        loc[XPath(prefix + MIN_VEL_THETA_ATTRIBUTE)]            = this->min_vel_theta;
        loc[XPath(prefix + MIN_VEL_X_ATTRIBUTE)]                = this->min_vel_x;
        loc[XPath(prefix + OCCDIST_SCALE_ATTRIBUTE)]            = this->occdist_scale;
        loc[XPath(prefix + OSCILLATION_RESET_DIST_ATTRIBUTE)]   = this->oscillation_reset_dist;
        loc[XPath(prefix + PDIST_SCALE_ATTRIBUTE)]              = this->pdist_scale;
        loc[XPath(prefix + RESTORE_DEFAULTS_ATTRIBUTE)]         = this->restore_defaults == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + SIMPLE_ATTRACTOR_ATTRIBUTE)]         = this->simple_attractor == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + SIM_GRANULARITY_ATTRIBUTE)]          = this->sim_granularity;
        loc[XPath(prefix + SIM_TIME_ATTRIBUTE)]                 = this->sim_time;
        loc[XPath(prefix + VTHETA_SAMPLES_ATTRIBUTE)]           = this->vtheta_samples;
        loc[XPath(prefix + VX_SAMPLES_ATTRIBUTE)]               = this->vx_samples;                
        loc[XPath(prefix + Y_VELS_ATTRIBUTE)]                   = this->y_vels;
    }    
    
    void ROSBaseLocalPlannerConfiguration::fromLocation(const xmltio::Location &loc, ROSBaseLocalPlannerConfiguration &config)
    {
        string prefix = BASELOCALPLANNER_CONFIGURATION_TAG 
                + ROSDynamicConfiguration::X_PATH_SEPARATOR 
                + ROSDynamicConfiguration::ATTRIBUTE_PREFIX;
	
        Type::fromLocation(loc, config);
        
        config.setAccLimTheta(          extract<double>(loc[XPath(prefix + ACC_LIM_THETA_ATTRIBUTE)]));
	config.setAccLimX(              extract<double>(loc[XPath(prefix + ACC_LIM_X_ATTRIBUTE)]));
        config.setAccLimY(              extract<double>(loc[XPath(prefix + ACC_LIM_Y_ATTRIBUTE)]));
        config.setAngularSimGranularity(extract<double>(loc[XPath(prefix + ANGULAR_SIM_GRANULARITY_ATTRIBUTE)]));

        string dwaString = extract<string>(loc[XPath(prefix + DWA_ATTRIBUTE)]);
        config.setDwa(dwaString.compare(TRUE_STRING) == 0 ? true : false);        
        
        config.setEscapeResetDist(      extract<double>(loc[XPath(prefix + ESCAPE_RESET_DIST_ATTRIBUTE)]));
        config.setEscapeResetTheta(     extract<double>(loc[XPath(prefix + ESCAPE_RESET_THETA_ATTRIBUTE)]));
        config.setEscapeVel(            extract<double>(loc[XPath(prefix + ESCAPE_VEL_ATTRIBUTE)]));
        config.setGdistScale(           extract<double>(loc[XPath(prefix + GDIST_SCALE_ATTRIBUTE)]));   
        config.setHeadingLookahead(     extract<double>(loc[XPath(prefix + HEADING_LOOKAHEAD_ATTRIBUTE)]));
        
        string headingScoringString = extract<string>(loc[XPath(prefix + HEADING_SCORING_ATTRIBUTE)]);
        config.setHeadingScoring(headingScoringString.compare(TRUE_STRING) == 0 ? true : false);
        
        config.setHeadingScoringTimestep(extract<double>(loc[XPath(prefix + HEADING_SCORING_TIMESTAMP_ATTRIBUTE)]));
        
        string holonomicRobotString = extract<string>(loc[XPath(prefix + HOLONOMIC_ROBOT_ATTRIBUTE)]);
        config.setHolonomicRobot(holonomicRobotString.compare(TRUE_STRING) == 0 ? true : false);
        
        config.setMaxVelTheta(          extract<double>(loc[XPath(prefix + MAX_VEL_THETA_ATTRIBUTE)]));
        config.setMaxVelX(              extract<double>(loc[XPath(prefix + MAX_VEL_X_ATTRIBUTE)]));
        config.setMinInPlaceVelTheta(   extract<double>(loc[XPath(prefix + MIN_IN_PLACE_VEL_THETA_ATTRIBUTE)]));
        config.setMinVelTheta(          extract<double>(loc[XPath(prefix + MIN_VEL_THETA_ATTRIBUTE)]));
        config.setMinVelX(              extract<double>(loc[XPath(prefix + MIN_VEL_X_ATTRIBUTE)]));
        config.setOccdistScale(         extract<double>(loc[XPath(prefix + OCCDIST_SCALE_ATTRIBUTE)]));
        config.setOscillationResetDist( extract<double>(loc[XPath(prefix + OSCILLATION_RESET_DIST_ATTRIBUTE)]));
        config.setPdistScale(           extract<double>(loc[XPath(prefix + PDIST_SCALE_ATTRIBUTE)]));

        string restoreDefaultsString = extract<string>(loc[XPath(prefix + RESTORE_DEFAULTS_ATTRIBUTE)]);
        config.setRestoreDefaults(restoreDefaultsString.compare(TRUE_STRING) == 0 ? true : false);        
        
        config.setSimGranularity(       extract<double>(loc[XPath(prefix + SIM_GRANULARITY_ATTRIBUTE)]));
        config.setSimTime(              extract<double>(loc[XPath(prefix + SIM_TIME_ATTRIBUTE)]));
        
        string simpleAttractorString = extract<string>(loc[XPath(prefix + SIMPLE_ATTRACTOR_ATTRIBUTE)]);
        config.setSimpleAttractor(simpleAttractorString.compare(TRUE_STRING) == 0 ? true : false); 
        
        config.setVthetaSamples(        extract<int>(loc[XPath(prefix + VTHETA_SAMPLES_ATTRIBUTE)]));
        config.setVxSamples(            extract<int>(loc[XPath(prefix + VX_SAMPLES_ATTRIBUTE)]));
        config.setYVels(                extract<string>(loc[XPath(prefix + Y_VELS_ATTRIBUTE)]));
    }    
}
