/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "TurnData.h"
#include "../../units/UnitConverter.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

RotationalSpeedUnit TurnData::iSU = RADIANS_PER_SEC;
AngleUnit TurnData::iLU = RADIAN;

std::string TurnData::getBaseTag() {
	return "TURN";
}

std::string TurnData::getTemplate() {
	return std::string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + MicroTimestamp::getTemplate()
			+ "<SPEED quant=\"\"/>" + "<ANGLE quant=\"\"/>"
			+ "</" + getBaseTag() + ">";
}

TurnData::TurnData() :
		Type("unknown") {
}

double TurnData::getSpeed(RotationalSpeedUnit u) const {
	return UnitConverter::convert(speed, iSU, u);
}

void TurnData::setSpeed(double speed, RotationalSpeedUnit u) {
	this->speed = UnitConverter::convert(speed, u, iSU);
}

double TurnData::getAngle(AngleUnit u) const {
	return UnitConverter::convert(angle, iLU, u);
}

void TurnData::setAngle(double angle, AngleUnit u) {
	this->angle = UnitConverter::convert(angle, u, iLU);
}

void TurnData::sanitizeLocation(xmltio::Location &loc) const {

}

void TurnData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();
	loc[XPath("ANGLE/@quant")] = getAngle(iLU);
	loc[XPath("SPEED/@quant")] = getSpeed(iSU);

}

void TurnData::fromLocation(const xmltio::Location &loc_, TurnData &sd) {

	Type::fromLocation(loc_, sd);

	sd.setAngle(extract<double>(loc_[XPath("ANGLE/@quant")]), iLU);
	sd.setSpeed(extract<double>(loc_[XPath("SPEED/@quant")]),
			iSU);

	sd.setReadTime(
			extract<MicroTimestamp>(loc_[XPath(MicroTimestamp::getBaseTag())]));

}
