/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "ObjectData.h"
#include <boost/make_shared.hpp>

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string ObjectData::getBaseTag() {
	return "OBJECT";
}

std::string ObjectData::Hypothesis::getBaseTag() {
	return "HYPOTHESIS";
}

std::string ObjectData::getTemplate() {
	return "<" + getBaseTag() + "></" + getBaseTag() + ">";
}

std::string ObjectData::Hypothesis::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<RATING><RELIABILITY value=\"\"/></RATING><CLASS/></" + getBaseTag()
			+ ">";
}

ObjectData::ObjectData() :
		Type("unknown") {
}

ObjectData::Hypothesis::Hypothesis() :
		Type("unknown") {
}

ObjectData::ObjectData(std::string generator) :
		Type(generator) {
}

ObjectData::Hypothesis::Hypothesis(std::string generator) :
		Type(generator) {
}

ObjectData::~ObjectData() {
}

ObjectData::Hypothesis::Hypothesis(const Hypothesis &h) :
		Type(), reliability(h.reliability), classLabel(h.classLabel) {

	setTimestamp(h.getTimestamp());
	setGenerator(h.getGenerator());
}

void ObjectData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(ObjectData::Hypothesis::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectData::Hypothesis::sanitizeLocation(xmltio::Location&) const {

}

void ObjectData::fillInto(xmltio::Location &loc) const {
	for (vector<boost::shared_ptr<Hypothesis> >::const_iterator it =
			hypotheses.begin(); it != hypotheses.end(); it++) {

		Hypothesis h(*it->get());
		Location hyp(Hypothesis::getTemplate(),
				XPath(Hypothesis::getBaseTag()));
		h.fillInto(hyp);
		loc.add(hyp);
	}
}

void ObjectData::Hypothesis::fillInto(xmltio::Location &loc) const {
	Type::fillInto(loc);
	loc[XPath("RATING/RELIABILITY/@value")] = reliability;
	loc[XPath("CLASS")] = classLabel;
}

void ObjectData::fromLocation(const xmltio::Location &loc_,
		ObjectData &object) {

	for (XPathIterator iter = XPath(Hypothesis::getBaseTag()).evaluate(loc_);
			iter; ++iter) {
		Hypothesis hyp;
		Hypothesis::fromLocation(*iter, hyp);
		object.addHypothesis(hyp);
	}

}

void ObjectData::Hypothesis::fromLocation(const xmltio::Location &loc_,
		Hypothesis &object) {

	Type::fromLocation(loc_, object);
	double r = extract<double>(loc_[XPath("RATING/RELIABILITY/@value")]);
	object.setReliability(r);

	string s = extract<string>(loc_[XPath("CLASS")]);
	object.classLabel = s;
}

string ObjectData::Hypothesis::getClassLabel() const {
	return classLabel;
}
void ObjectData::Hypothesis::setClassLabel(const string label) {
	this->classLabel = label;
}

double ObjectData::Hypothesis::getReliability() const {
	return reliability;
}
void ObjectData::Hypothesis::setReliability(const double reliability) {
	this->reliability = reliability;
}

bool ObjectData::operator==(const ObjectData &other) const {

	bool isHyp = true;

	vector<Hypothesis> h0 = getHypotheses();
	vector<Hypothesis> h1 = other.getHypotheses();

	vector<Hypothesis>::iterator it0 = h0.begin();
	for (vector<Hypothesis>::iterator it1 = h1.begin(); it1 != h1.end();
			it1++) {

		Hypothesis hyp0 = *it0;
		Hypothesis hyp1 = *it1;
		if (!(hyp0 == hyp1)) {
			isHyp = false;
		}
		it0++;
	}

	return isHyp;
}

bool ObjectData::Hypothesis::operator==(const Hypothesis &other) const {
	return other.getGenerator() == getGenerator()
			&& other.getTimestamp() == getTimestamp()
			&& other.getReliability() == getReliability()
			&& other.getClassLabel() == getClassLabel();
}

void ObjectData::addHypothesis(Hypothesis &h) {
	hypotheses.push_back(boost::shared_ptr<Hypothesis>(new Hypothesis(h)));
}

void ObjectData::clearHypotheses() {
	hypotheses.clear();
}

vector<ObjectData::Hypothesis> ObjectData::getHypotheses() const {
	vector<ObjectData::Hypothesis> h;
	for (vector<boost::shared_ptr<ObjectData::Hypothesis> >::const_iterator it =
			hypotheses.begin(); it != hypotheses.end(); it++) {
		h.push_back(*(it->get()));
	}
	return h;
}
