/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "ObjectShapeData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

LengthUnit ObjectShapeData::internalUnit = MILLIMETER;

std::string ObjectShapeData::getBaseTag() {
	return "OBJECTSHAPE";
}

std::string ObjectShapeData::Hypothesis::getBaseTag() {
	return "HYPOTHESIS";
}

std::string ObjectShapeData::getTemplate() {
	return "<" + getBaseTag()
			+ "><SENSORTYPE/><POLYGON/><SHAPE width=\"\" height=\"\" depth=\"\" />"
			+ Point3D::getTemplate() + "</" + getBaseTag() + ">";
}

std::string ObjectShapeData::Hypothesis::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<RATING><RELIABILITY value=\"\"/></RATING><CLASS/></"
			+ getBaseTag() + ">";
}

ObjectShapeData::ObjectShapeData() :
		ObjectLocationData() {
}

ObjectShapeData::Hypothesis::Hypothesis() :
		ObjectLocationData::Hypothesis() {
}

ObjectShapeData::ObjectShapeData(std::string generator) :
		ObjectLocationData(generator) {
}

ObjectShapeData::Hypothesis::Hypothesis(std::string generator) :
		ObjectLocationData::Hypothesis(generator) {
}

void ObjectShapeData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(Hypothesis::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectShapeData::Hypothesis::sanitizeLocation(
		xmltio::Location &loc) const {
	XPath xp("POINT");
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectShapeData::fillInto(xmltio::Location &loc) const {

	ObjectLocationData::fillInto(loc);
	ObjectLocationData::sanitizeLocation(loc);

	loc[XPath("SHAPE/@width")] = width;
	loc[XPath("SHAPE/@height")] = height;
	loc[XPath("SHAPE/@depth")] = depth;
	Location cen = loc[XPath(Point3D::getBaseTag())];
	center.fillInto(cen);

	for (vector<boost::shared_ptr<ObjectData::Hypothesis> >::const_iterator it =
			hypotheses.begin(); it != hypotheses.end(); it++) {

		try {
			Hypothesis* h0 = dynamic_cast<Hypothesis*>(it->get());

Location			hyp(Hypothesis::getTemplate(), XPath(
							Hypothesis::getBaseTag()));

			h0->fillInto(hyp);
			loc.add(hyp);
		} catch (bad_cast &e) {
			continue;
		}
	}
}

void ObjectShapeData::Hypothesis::fillInto(xmltio::Location &loc) const {

	ObjectLocationData::Hypothesis::fillInto(loc);

}

void ObjectShapeData::fromLocation(const xmltio::Location &loc_,
		ObjectShapeData &object) {

	ObjectLocationData::fromLocation(loc_, object);
	object.clearHypotheses();

	object.width = extract<double>(loc_[XPath("SHAPE/@width")]);
	object.height = extract<double>(loc_[XPath("SHAPE/@height")]);
	object.depth = extract<double>(loc_[XPath("SHAPE/@depth")]);
	object.center = extract<Point3D>(loc_[XPath(Point3D::getBaseTag())]);

	for (XPathIterator iter = XPath(Hypothesis::getBaseTag()).evaluate(loc_);
			iter; ++iter) {
		Hypothesis hyp;
		Hypothesis::fromLocation(*iter, hyp);
		object.addHypothesis(hyp);
	}
}

void ObjectShapeData::Hypothesis::fromLocation(const xmltio::Location &loc_,
		Hypothesis &object) {

	ObjectLocationData::Hypothesis::fromLocation(loc_, object);

}

bool ObjectShapeData::operator==(const ObjectShapeData &other) const {

	bool isPoly = true;
	if (other.getPolygon().size() != getPolygon().size()) {
		isPoly = false;
	} else {
		for (int i = 0; i < (int) getPolygon().size(); i++) {
			if (getPolygon()[i].first != other.getPolygon()[i].first
					|| getPolygon()[i].second != other.getPolygon()[i].second) {
				isPoly = false;
			}
		}
	}

	bool isHyp = true;

	try {
		vector<Hypothesis> h0 = getShapeHypotheses();
		vector<Hypothesis> h1 = other.getShapeHypotheses();

		if (h0.size() != h1.size()) {
			return false;
		}

		vector<Hypothesis>::iterator it0 = h0.begin();
		for (vector<Hypothesis>::iterator it1 = h1.begin(); it1 != h1.end();
				it1++) {

			Hypothesis hyp0 = *it0;
			Hypothesis hyp1 = *it1;

			if (!(hyp0 == hyp1)) {
				isHyp = false;
			}
			it0++;
		}

		return ObjectData::operator==(other) && isHyp && isPoly
				&& other.getSensorType() == getSensorType();
	} catch (bad_cast &e) {
		return false;
	}
	return false;
}

bool ObjectShapeData::Hypothesis::operator==(const Hypothesis &other) const {

	return ObjectLocationData::Hypothesis::operator==(other);
}

void ObjectShapeData::addHypothesis(ObjectShapeData::Hypothesis &h) {
	hypotheses.push_back(boost::shared_ptr<Hypothesis>(new Hypothesis(h)));
}

vector<ObjectShapeData::Hypothesis> ObjectShapeData::getShapeHypotheses() const {

	vector<Hypothesis> h1;

	for (vector<boost::shared_ptr<ObjectData::Hypothesis> >::const_iterator it =
			hypotheses.begin(); it != hypotheses.end(); it++) {
		try {
			Hypothesis* hyp = dynamic_cast<Hypothesis*>(it->get());h1.push_back(*hyp);
		} catch (bad_cast &e) {
				cerr << "Cannot cast Hypothesis" << endl;
				continue;
			}
		}
	return h1;
}

Point3D ObjectShapeData::getCenter() const {
	return center;
}

void ObjectShapeData::setCenter(Point3D center) {
	this->center = center;
}

double airobots::ObjectShapeData::getDepth(LengthUnit unit) const {
	return UnitConverter::convert(depth, internalUnit, unit);
}

void airobots::ObjectShapeData::setDepth(double depth, LengthUnit unit) {
	this->depth = UnitConverter::convert(depth, unit, internalUnit);
}

double airobots::ObjectShapeData::getHeight(LengthUnit unit) const {
	return UnitConverter::convert(height, internalUnit, unit);
}

void airobots::ObjectShapeData::setHeight(double height, LengthUnit unit) {
	this->height = UnitConverter::convert(height, unit, internalUnit);
}

double airobots::ObjectShapeData::getWidth(LengthUnit unit) const {
	return UnitConverter::convert(width, internalUnit, unit);
}

void airobots::ObjectShapeData::setWidth(double width, LengthUnit unit) {
	this->width = UnitConverter::convert(width, unit, internalUnit);
}

