/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef OBJECTSHAPEDATA_H_
#define OBJECTSHAPEDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "../../Type.h"
#include "ObjectLocationData.h"
#include "../geometry/Point3D.h"

namespace airobots {

/**
 * Results of the object recognition.
 *
 * @author lziegler
 */
class ObjectShapeData: public ObjectLocationData {

public:

	class Hypothesis: public ObjectLocationData::Hypothesis {
	private:

	public:

		static std::string getBaseTag();

		static std::string getTemplate();

		Hypothesis();
		Hypothesis(std::string generator);

		virtual bool operator==(const Hypothesis &other) const;

		void sanitizeLocation(xmltio::Location&) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, Hypothesis &type);

	};

	static std::string getBaseTag();

	static std::string getTemplate();

	ObjectShapeData();
	ObjectShapeData(std::string generator);

	void addHypothesis(Hypothesis &h);
	std::vector<Hypothesis> getShapeHypotheses() const;

	virtual bool operator==(const ObjectShapeData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			ObjectShapeData &type);
	Point3D getCenter() const;
	void setCenter(Point3D center);
	double getDepth(LengthUnit unit) const;
	void setDepth(double depth, LengthUnit unit);
	double getHeight(LengthUnit unit) const;
	void setHeight(double depth, LengthUnit unit);
	double getWidth(LengthUnit unit) const;
	void setWidth(double depth, LengthUnit unit);
private:
	std::vector<ObjectData::Hypothesis> getHypotheses() const;
	double depth, height, width;
	Point3D center;
	static LengthUnit internalUnit;
};

}

#endif /* OBJECTLOCATIONDATA_H_ */
