/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef BODY_H_
#define BODY_H_

#include "../../Type.h"
#include "../person/Percept.h"
#include "../geometry/Point3D.h"
#include "../../units/UnitConverter.h"

namespace airobots {

/**
 * Body percept.
 *
 * @author Dennis Leroy Wigand (dwigand)
 */
class Body: public Percept {

public:

	static std::string getBaseTag();
	static std::string getTemplate();
	static std::string getPerceptName();

	Body();
	virtual ~Body();

	Point3D getCoM() const;
	void setCoM(Point3D com);

	Point3D getTorso() const;
	void setTorso(Point3D joint);

	bool isSkeleton() const;
	void setSkeleton(bool isSkeleton);

	int getKinectId() const;
	void setKinectId(int id);

	float getAngle(AngleUnit angleUnit) const;

	float getDistance(LengthUnit lengthUnit) const;
	void setDistanceAndAngle(float distance, LengthUnit lengthUnit, float angle, AngleUnit angleUnit);

	bool operator==(const Body &other) const;

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Body &type);

private:

	bool isSkeletonActive;
	Point3D centerOfMass;
	Point3D skel_Torso;
	int kinect_id;

};

std::ostream& operator<<(std::ostream& o, Body const& body);

}

#endif /* BODY_H_ */
