/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/** ==========================================================================
 * 
 * @file		SplocAngleData.cpp --
 * @brief		Source file for the angle output of the sploc application.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * @author  dklotz
 * 
 * ========================================================================== 
 **/

#include "SplocAngleData.h"

using namespace airobots;
using namespace xmltio;

SplocAngleData::SplocAngleData() {

}

SplocAngleData::~SplocAngleData() {

}

std::string SplocAngleData::getBaseTag() {
	return "SPLOCANGLES";
}

std::string SplocAngleData::getTemplate() {

	std::stringstream tmp;
	tmp << "<" << getBaseTag() << ">" << "<GENERATOR/>"
			<< Timestamp::getTemplate() << MicroTimestamp::getTemplate()
			<< "<ANGLES/>" << "</" << getBaseTag() << ">";

	return tmp.str();

}

std::vector<float> SplocAngleData::getAngles() {
	return angles;
}

void SplocAngleData::setAngles(std::vector<float> angles) {
	this->angles = angles;
}

void SplocAngleData::sanitizeLocation(xmltio::Location &loc) const {

}

void SplocAngleData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();

	Location anglesLoc = loc[XPath("ANGLES")];

	for (unsigned int i = 0; i < angles.size(); i++) {
		Location angleLoc("<ANGLE value=\"\"/>", "/ANGLE");

		angleLoc[XPath("@value")] = angles[i];

		anglesLoc.add(angleLoc);
	}

}

void SplocAngleData::fromLocation(const xmltio::Location &loc,
		SplocAngleData &data) {

	Type::fromLocation(loc, data);

	std::vector<float> angles;

	for (XPathIterator it = XPath("ANGLES/ANGLE").evaluate(loc); it; ++it) {
		angles.push_back(xmltio::extract<float>((*it)[XPath("@value")]));
	}

	data.setAngles(angles);

	data.setReadTime(
			extract<MicroTimestamp> (loc[XPath(MicroTimestamp::getBaseTag())]));
}
