/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * StallData.cpp
 *
 *  Created on: Jun 13, 2011
 *      Author: lziegler
 */

#include "StallData.h"
#include <algorithm>

using namespace std;
using namespace xmltio;
using namespace airobots;

string StallData::getBaseTag() {
	return "STALL";
}

string StallData::getTemplate() {
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + MicroTimestamp::getTemplate()
			+ "<stalled/>" + "</" + getBaseTag() + ">";
}

StallData::StallData() :
		Type("unknown") {

}

bool StallData::isStalled() const {
	return stalled;
}

void StallData::setStalled(bool stalled) {
	this->stalled = stalled;
}

void StallData::sanitizeLocation(xmltio::Location &loc) const {

}

void StallData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();
	if (isStalled()) {
		loc[XPath("stalled")] = "true";
	} else {
		loc[XPath("stalled")] = "false";
	}

}

void StallData::fromLocation(const xmltio::Location &loc_, StallData &sd) {

	Type::fromLocation(loc_, sd);

	string stallString = loc_[XPath("stalled")].getTextValue();
	transform(stallString.begin(), stallString.end(), stallString.begin(),
			::tolower);
	sd.setStalled(stallString == "true");

	sd.setReadTime(
			extract<MicroTimestamp>(loc_[XPath(MicroTimestamp::getBaseTag())]));

}
