/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "StateConfiguration.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

const string StateConfiguration::STATE_ELEMENT = "STATE";
const string StateConfiguration::SYSTEM_STATES_ELEMENT = "SYSTEMSTATES";
const string StateConfiguration::SYSTEM_NAME_ELEMENT = "SYSTEMNAME";

StateConfiguration::StateConfiguration() {
	systemName = "unknown";
	states = set<string>();
}

StateConfiguration::~StateConfiguration() {
}

void airobots::StateConfiguration::addStates(std::set<std::string> stateNames)
{
	states.insert(stateNames.begin(), stateNames.end());
}



const std::string airobots::StateConfiguration::getSystemName() const
{
	return systemName;
}



std::string airobots::StateConfiguration::getBaseTag()
{
	return "STATEVIZCONFIG";
}



std::string airobots::StateConfiguration::getTemplate()
{
	return "<" + getBaseTag() + ">"
		+ "<GENERATOR/>"
		+ Timestamp::getTemplate()
		+ "<" + SYSTEM_NAME_ELEMENT + "/>"
		+ "<" + SYSTEM_STATES_ELEMENT + ">"
		+ "</" + SYSTEM_STATES_ELEMENT + ">"
		+ "</" + getBaseTag() + ">";
}



void airobots::StateConfiguration::addState(std::string stateName)
{
	states.insert(stateName);
}



void airobots::StateConfiguration::setSystemName(std::string name)
{
	systemName = name;
}



void airobots::StateConfiguration::fromLocation(const xmltio::Location & loc, StateConfiguration & sc)
{
	Type::fromLocation(loc, sc);

	string name = extract<string>(loc[XPath(SYSTEM_NAME_ELEMENT)]);
	sc.setSystemName(name);

	Location statesLoc = loc[XPath(SYSTEM_STATES_ELEMENT)];
	for (XPathIterator i = XPath(STATE_ELEMENT).evaluate(statesLoc); i; ++i) {
		Location stateLoc(*i);
		string value = extract<string>(stateLoc);
		sc.addState(value);
	}
}



void airobots::StateConfiguration::removeAllStates()
{
	states.clear();
}



void airobots::StateConfiguration::sanitizeLocation(xmltio::Location & loc) const
{
}



void airobots::StateConfiguration::fillInto(xmltio::Location & loc) const
{
	Type::fillInto(loc);

	loc[XPath(SYSTEM_NAME_ELEMENT)] = getSystemName();

	set<string>::iterator i = states.begin();
	for (; i != states.end(); ++i) {
		Location stateLoc("<" + STATE_ELEMENT + "/>", STATE_ELEMENT);
		stateLoc = *i;
		loc[XPath(SYSTEM_STATES_ELEMENT)].add(stateLoc);
	}
}



std::set<std::string> airobots::StateConfiguration::getStates()
{
	return states;
}


