/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * SonarData.cpp
 *
 *  Created on: Nov 4, 2010
 *      Author: ltwardon
 */

#include "SonarData.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

std::string SonarData::getBaseTag()
{
	return "SONARDATA";
}

std::string SonarData::getTemplate()
{
	return std::string("<") + getBaseTag() + "><GENERATOR/>"
			+ Timestamp::getTemplate() + "<DISTLEFT value=\"\" />"
			+ "<DISTRIGHT value=\"\" />"
			+ "</" + getBaseTag() + ">";
}

SonarData::SonarData() :
	Type("unknown")
{

}

SonarData::SonarData(double distanceRight, double distanceLeft, LengthUnit lU)
{
	this->distanceRight = UnitConverter::convert(distanceRight,lU,MILLIMETER);
	this->distanceLeft = UnitConverter::convert(distanceLeft,lU,MILLIMETER);
}

double SonarData::getDistanceRight(LengthUnit lengthUnit) const
{
	return UnitConverter::convert(distanceRight,MILLIMETER,lengthUnit);
}
void SonarData::setDistanceRight(double distanceRight,LengthUnit lengthUnit)
{
	this->distanceRight = UnitConverter::convert(distanceRight,lengthUnit,MILLIMETER);
}

double SonarData::getDistanceLeft(LengthUnit lengthUnit) const
{
	return UnitConverter::convert(distanceLeft,MILLIMETER,lengthUnit);
}
void SonarData::setDistanceLeft(double distanceLeft,LengthUnit lengthUnit)
{
	this->distanceLeft = UnitConverter::convert(distanceLeft,lengthUnit,MILLIMETER);
}

void SonarData::sanitizeLocation(xmltio::Location &loc) const
{

}

void SonarData::fillInto(xmltio::Location &loc) const
{

	Type::fillInto(loc);

	loc[XPath("DISTLEFT/@value")] = getDistanceLeft(METER);
	loc[XPath("DISTRIGHT/@value")] = getDistanceRight(METER);

}

void SonarData::fromLocation(const xmltio::Location &loc_, SonarData &sd)
{

	Type::fromLocation(loc_, sd);

	sd.setDistanceLeft(extract<double> (loc_[XPath("DISTLEFT/@value")]),METER);
	sd.setDistanceRight(extract<double> (loc_[XPath("DISTRIGHT/@value")]),METER);

}

