/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "RegionData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string RegionData::getBaseTag() {
	return "REGION";
}

std::string RegionData::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR />" + Timestamp::getTemplate()
			+ "<SCOPE/>" + PrecisePolygon::getTemplate() + "</" + getBaseTag() + ">";
}

RegionData::RegionData() :
	Type() {
}

RegionData::RegionData(std::string generator) :
	Type(generator) {
}

void RegionData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp("POINT2D");
	while (XPathIterator i = xp.evaluate(loc[XPath("PRECISEPOLYGON")])) {
		Location l(*i);
		loc.remove(l);
	}
}

void RegionData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);
	Location polyLoc = Location(loc, "PRECISEPOLYGON");
	if (scope == LOCAL) {
		loc[XPath("SCOPE")] = "LOCAL";
	} else {
		loc[XPath("SCOPE")] = "GLOBAL";
	}
	loc[XPath("PRECISEPOLYGON")] = polygon;
}

void RegionData::fromLocation(const xmltio::Location &loc_, RegionData &object) {

	Type::fromLocation(loc_, object);

	string s = extract<string> (loc_[XPath("SCOPE")]);
	if (s == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}

	PrecisePolygon poly = extract<PrecisePolygon>(loc_[XPath("PRECISEPOLYGON")]);
	object.setPolygon(poly);
}

vector<pair<double, double> > RegionData::scalePolygon(vector<pair<double,
		double> > poly, double factor) const {
	for (vector<pair<double, double> >::iterator it = poly.begin(); it
			!= poly.end(); ++it) {
		it->first *= factor;
		it->second *= factor;
	}
	return poly;
}

vector<pair<double, double> > RegionData::getPolygon(LengthUnit unit) const {
	vector<pair<double, double> > poly;
	vector<Point2D> points = polygon.getPoints();
	for (vector<Point2D>::const_iterator i = points.begin(); i != points.end(); ++i) {
		poly.push_back(pair<double, double>(i->getX(unit), i->getY(unit)));
	}

	return poly;

}
void RegionData::setPolygon(const vector<pair<double, double> > &poly,
		LengthUnit unit) {
	vector<pair<double, double> > polyTmp(poly);

	PrecisePolygon newPoly;
	for (vector<pair<double, double> >::iterator i = polyTmp.begin();
			i != polyTmp.end();
			++i) {
		double x = i->first;
		double y = i->second;
		newPoly.addPoint(x, y, unit);
	}

	setPolygon(newPoly);
}

RegionData::Scope RegionData::getScope() const {
	return scope;
}

void RegionData::setScope(const Scope &scope) {
	this->scope = scope;
}

bool RegionData::operator==(const RegionData &other) const {

	return scope == other.scope && polygon == other.getPolygon();
}

PrecisePolygon RegionData::getPolygon() const {
	return polygon;
}

void RegionData::setPolygon(PrecisePolygon poly) {
	polygon = poly;
}
