package de.unibi.airobots.btl.data.annotation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.annotation.MapPoint.Kind;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.data.object.ObjectData;
import de.unibi.airobots.btl.data.object.ObjectData.Hypothesis;
import de.unibi.airobots.btl.data.person.Face;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.util.LinkedList;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;
import de.unibi.airobots.btl.data.map.Annotation;

/**
 * 
 * @author jhelbach
 * 
 * @deprecated Use {@link Annotation} instead.
 */
@Deprecated
public class MapRegion extends Type {

	private static final String HEIGHT_TAG = "HEIGHT";
	private static final String CONFIDENCE_TAG = "CONFIDENCE";
	private static final String REGIONGROUP_TAG = "REGIONGROUP";
	private static final String REGIONLABEL_TAG = "REGIONLABEL";
	private static final String REGIONTYPE_TAG = "REGIONTYPE";
	private final static String BASETAG = "MAPREGION";
	public final static String POLYGONE_TYPE = "polygon";
	public final static String RECTANGLE_TYPE = "rectangle";
	private String type = "";
	private String label = "";
	private String group = "";
	private double confidence = 1.0;
	private double height = 0;
	private LinkedList<MapPoint> points = new LinkedList<MapPoint>();
	private LinkedList<NavigationGoalData> viewPoints = new LinkedList<NavigationGoalData>();
	private Logger logger = Logger.getLogger(this.getClass());
	private LengthUnit iLU = LengthUnit.METER;

	public MapRegion() {

	}

	public MapRegion(String type) {
		this.type = type;
	}

	public MapRegion(String type, String label) {
		this.type = type;
		this.label = label;
	}

	public MapRegion(String type, String label, String group) {
		this.type = type;
		this.label = label;
		this.group = group;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoints.add(viewPoint);
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points,
			LinkedList<NavigationGoalData> viewPoints) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoints = viewPoints;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint,
			double height) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoints.add(viewPoint);
		this.height = height;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points,
			LinkedList<NavigationGoalData> viewPoints, double height) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoints = viewPoints;
		this.height = height;
	}

	public void addPoint(int x, int y, LengthUnit lU) {
		MapPoint p = new MapPoint((double) x, (double) y, lU);
		points.add(p);
	}

	public void addPoint(double x, double y, LengthUnit lU) {
		MapPoint p = new MapPoint(x, y, lU);
		points.add(p);
	}

	public void addPoint(MapPoint point) {
		points.add(point);
	}

	public Shape getShape() {
		GeneralPath generalPath = new GeneralPath();

		boolean firstPoint = true;
		logger.debug("points:");
		for (MapPoint p : points) {
			if (firstPoint) {
				generalPath.moveTo(p.getX(iLU), p.getY(iLU));
				firstPoint = false;
			} else {
				generalPath.lineTo(p.getX(iLU), p.getY(iLU));
				logger.debug("x " + (int) p.getX(iLU) + " y "
						+ (int) p.getY(iLU));
			}
		}

		return generalPath;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public void setConfidence(double confidence) {
		this.confidence = confidence;
	}

	public double getConfidence() {
		return confidence;
	}

	public LinkedList<MapPoint> getPoints() {
		return points;
	}

	public void setPoints(LinkedList<MapPoint> points) {
		this.points = points;
	}

	public LinkedList<NavigationGoalData> getViewPoints() {
		return viewPoints;
	}

	public NavigationGoalData getNearestViewPoint(PositionData position) {
		double minDist = Double.MAX_VALUE;
		NavigationGoalData result = null;
		for (NavigationGoalData navData : viewPoints) {
			double distance = Math.sqrt(Math.pow(navData.getX(LengthUnit.METER)
					- position.getX(LengthUnit.METER), 2)
					+ Math.pow(
							navData.getY(LengthUnit.METER)
									- position.getY(LengthUnit.METER), 2));
			if (distance < minDist) {
				minDist = distance;
				result = navData;
			}

		}
		return result;
	}

	/**
	 * 
	 * The first added view point.
	 * 
	 * @return First added view point.
	 * 
	 * @Deprecated Since 0.10.1 a MapRegion can have more than one view point.
	 *             Use {@link getFirstViewPoint()} or {@link getLastViewPoint()}
	 *             or {@link getFViewPoints()} instead.
	 */
	@Deprecated
	public NavigationGoalData getViewPoint() {
		return getFirstViewPoint();
	}

	/**
	 * The first added view point.
	 * 
	 * @return First added view point.
	 */
	public NavigationGoalData getFirstViewPoint() {
		if (viewPoints.isEmpty()) {
			return null;
		}
		return viewPoints.getFirst();
	}

	/**
	 * The last added view point.
	 * 
	 * @return Last added point.
	 */
	public NavigationGoalData getLastViewPoint() {
		if (viewPoints.isEmpty()) {
			return null;
		}
		return viewPoints.getLast();
	}

	public void removeViewPoints() {
		viewPoints.clear();
	}

	public void setViewPoints(LinkedList<NavigationGoalData> viewPoints) {
		this.viewPoints = viewPoints;
	}

	public void addViewPoint(NavigationGoalData viewPoint) {
		viewPoints.add(viewPoint);
	}

	public double getHeight(LengthUnit lU) {
		return UnitConverter.convert(height, iLU, lU);
	}

	public void setHeight(double height, LengthUnit lU) {
		this.height = UnitConverter.convert(height, iLU, lU);
	}

	public void removePoint(double x, double y) {

		LinkedList<MapPoint> copy = new LinkedList<MapPoint>(points);

		for (MapPoint p : copy) {

			if (p.getX(iLU) == x && p.getY(iLU) == y) {
				points.remove(p);
			}
		}

	}

	public int getPointCount() {
		return points.size();
	}

	/**
	 * Returns {@link PositionData} for a Region. If one ViewPoint is set, the
	 * first ViewPoint will be returned. Otherwise the center of the bounding
	 * box.
	 * 
	 * @return
	 */
	public PositionData getPositionData() {

		if (!viewPoints.isEmpty()) {
			return viewPoints.getFirst();
		}

		Rectangle2D rect = getShape().getBounds2D();

		return new PositionData(rect.getCenterX(), rect.getCenterY(), 0,
				System.currentTimeMillis(), LengthUnit.METER, AngleUnit.RADIAN);
	}

	public boolean contains(MapRegion region) {
		if (region == null) {
			return false;
		}

		Rectangle2D pr = region.getShape().getBounds2D();

		return getShape().contains(pr);
	}

	public boolean contains(double x, double y) {
		return getShape().contains(x, y);

	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return BASETAG;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void sanitizeElement(Element parent) {
		super.sanitizeElement(parent);

		Nodes nodes = parent.query(MapPoint.getBaseTag());
		for (int i = 0; i < nodes.size(); i++) {
			parent.removeChild(nodes.get(i));
		}

		nodes = parent.query(NavigationGoalData.getBaseTag());
		for (int i = 0; i < nodes.size(); i++) {
			parent.removeChild(nodes.get(i));
		}
	}

	/**
	 * Fills an {@link ObjectData} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectData} object to fill with all the information
	 *            given by the {@link Element} object.
	 */
	public static void fromElement(Element objectElement, MapRegion type) {

		Type.fromElement(objectElement, type);

		try {

			Element g = objectElement.getFirstChildElement(REGIONGROUP_TAG);
			Element l = objectElement.getFirstChildElement(REGIONLABEL_TAG);
			Element t = objectElement.getFirstChildElement(REGIONTYPE_TAG);
			Element c = objectElement.getFirstChildElement(CONFIDENCE_TAG);
			Element h = objectElement.getFirstChildElement(HEIGHT_TAG);

			type.group = g.getValue();
			type.label = l.getValue();
			type.type = t.getValue();
			type.confidence = Double.parseDouble(c.getValue());
			type.height = Double.parseDouble(h.getValue());

			Nodes nodes = objectElement.query(MapPoint.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);

				if (node instanceof Element) {

					Element elem = (Element) node;

					MapPoint p = new MapPoint();
					MapPoint.fromElement(elem, p);
					type.addPoint(p);
				}
			}

			nodes = objectElement.query(NavigationGoalData.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);

				if (node instanceof Element) {

					Element elem = (Element) node;
					NavigationGoalData p = new NavigationGoalData();
					NavigationGoalData.fromElement(elem, p);
					type.addViewPoint(p);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		} catch (ParsingException e) {
			// TODO Auto-generated catch block
			throw new IllegalArgumentException("Parsing error", e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		sanitizeElement(parent);

		super.fillInto(parent);

		Element typeName = new Element(REGIONTYPE_TAG);
		typeName.appendChild(type);
		Element labelName = new Element(REGIONLABEL_TAG);
		labelName.appendChild(label);
		Element groupName = new Element(REGIONGROUP_TAG);
		groupName.appendChild(group);
		Element confName = new Element(CONFIDENCE_TAG);
		confName.appendChild(String.valueOf(confidence));
		Element heightName = new Element(HEIGHT_TAG);
		heightName.appendChild(String.valueOf(height));

		parent.appendChild(typeName);
		parent.appendChild(labelName);
		parent.appendChild(groupName);
		parent.appendChild(confName);
		parent.appendChild(heightName);

		for (MapPoint p : points) {

			Element pElement = new Element(MapPoint.getBaseTag());
			p.fillInto(pElement);

			parent.appendChild(pElement);

		}

		for (NavigationGoalData p : viewPoints) {

			Element pElement = new Element(NavigationGoalData.getBaseTag());
			p.fillInto(pElement);

			parent.appendChild(pElement);

		}
	}

	@Override
	public String toString() {
		return "MapRegion [type=" + type + ", label=" + label + ", group="
				+ group + ", confidence=" + confidence + ", height=" + height
				+ ", points=" + points + ", viewPoints=" + viewPoints
				+ ", logger=" + logger + ", iLU=" + iLU + "]";
	}

}
