package de.unibi.airobots.btl.data.command;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.speechrec.GrammarNonTerminal;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;

/**
 *
 * @author gminareci, ikillman
 */
public class CommandData extends Type {

    private GrammarNonTerminal tree = new GrammarNonTerminal();
    private String action = "";
    //default werte:
    private String location = "location";
    private String object = "object";
    private String person = "person";

    /**
     * @return the action
     */
    public String getAction() {
        return action;
    }

    /**
     * @param action the action to set
     */
    public void setAction(String action) {
        this.action = action;
    }

    /**
     * @return the location
     */
    public String getLocation() {
        return location;
    }

    /**
     * @param location the location to set
     */
    public void setLocation(String location) {
        this.location = location;
    }

    /**
     * @return the object
     */
    public String getObject() {
        return object;
    }

    /**
     * @param object the object to set
     */
    public void setObject(String object) {
        this.object = object;
    }

    /**
     * @return the tree
     */
    public GrammarNonTerminal getTree() {
        return tree;
    }

    /**
     * @param tree the tree to set
     */
    public void setTree(GrammarNonTerminal tree) {
        this.tree = tree;
    }
    
    /**
     * @return the person
     */
    public String getPerson() {
        return person;
    }

    /**
     * @param person the person to set
     */
    public void setPerson(String person) {
        this.person = person;
    }

    public static String getBaseTag() {
        return "COMMANDDATA";
    }

    public static void fromElement(Element element, CommandData commandData)
            throws ParsingException {

        Type.fromElement(element, commandData);

        String act = element.getFirstChildElement("ACTION").getValue();
        String loc = element.getFirstChildElement("LOCATION").getValue();
        String obj = element.getFirstChildElement("OBJECT").getValue();
        String per = element.getFirstChildElement("PERSON").getValue();

        GrammarNonTerminal tr = new GrammarNonTerminal();
        Element treeElement = element.getFirstChildElement("TREE");
        Element grammarElement = treeElement
                .getFirstChildElement(GrammarNonTerminal.getBaseTag());
        GrammarNonTerminal.fromElement(grammarElement, tr);

        commandData.setAction(act);
        commandData.setLocation(loc);
        commandData.setObject(obj);
        commandData.setPerson(per);
        commandData.setTree(tr);

    }

    @Override
    public void fillInto(Element parent) {

        super.fillInto(parent);

        Elements acts = parent.getChildElements("ACTION");
        for (int i = 0; i < acts.size(); i++) {
            parent.removeChild(acts.get(i));
        }
        Elements locs = parent.getChildElements("LOCATION");
        for (int i = 0; i < locs.size(); i++) {
            parent.removeChild(locs.get(i));
        }
        Elements objs = parent.getChildElements("OBJECT");
        for (int i = 0; i < objs.size(); i++) {
            parent.removeChild(objs.get(i));
        }
        Elements trs = parent.getChildElements("TREE");
        for (int i = 0; i < trs.size(); i++) {
            parent.removeChild(trs.get(i));
        }
        Elements pers = parent.getChildElements("PERSON");
        for (int i = 0; i < pers.size(); i++) {
            parent.removeChild(pers.get(i));
        }


        if (parent.getChildElements("ACTION").size() == 0) {
            Element e = new Element("ACTION");
            parent.appendChild(e);
        }

        if (parent.getChildElements("LOCATION").size() == 0) {
            Element e = new Element("LOCATION");
            parent.appendChild(e);
        }

        if (parent.getChildElements("OBJECT").size() == 0) {
            Element e = new Element("OBJECT");
            parent.appendChild(e);
        }
        
        if (parent.getChildElements("PERSON").size() == 0) {
            Element e = new Element("PERSON");
            parent.appendChild(e);
        }

        parent.getFirstChildElement("ACTION").appendChild(action);
        parent.getFirstChildElement("LOCATION").appendChild(location);
        parent.getFirstChildElement("OBJECT").appendChild(object);
        parent.getFirstChildElement("PERSON").appendChild(person);

        Element treeE1 = new Element("TREE");
        Element treeElP = new Element(GrammarNonTerminal.getBaseTag());
        tree.fillInto(treeElP);
        treeE1.appendChild(treeElP);
        parent.appendChild(treeE1);
    }
}
