package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.object.ObjectLocationData;

public class Path3DData extends Type {

	public static final String SCOPE_TAG_NAME = "SCOPE";

	public enum Scope {
		LOCAL, GLOBAL
	};

	/**
	 * Position of the object in the sensor's image.
	 */
	private PrecisePolygon3D polygon = new PrecisePolygon3D();

	private Scope scope = Scope.GLOBAL;

	public Path3DData() {
	}

	public Path3DData(PrecisePolygon3D polygon) {
		this.polygon = polygon;
	}

	public Path3DData(PrecisePolygon3D polygon, Scope scope) {
		this.polygon = polygon;
		this.scope = scope;
	}

	public Path3DData(Scope scope) {
		this.scope = scope;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PATH3D";
	}

	public PrecisePolygon3D getPolygon() {
		return polygon;
	}

	public void setPolygon(PrecisePolygon3D polygon) {
		this.polygon = polygon;
	}

	public Scope getScope() {
		return scope;
	}

	public void setScope(Scope scope) {
		this.scope = scope;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// polygon
		Element polygonElem = new Element(PrecisePolygon3D.getBaseTag());
		polygon.fillInto(polygonElem);

		Element scopeElem = new Element(SCOPE_TAG_NAME);
		scopeElem.appendChild(scope.name());

		parent.appendChild(polygonElem);
		parent.appendChild(scopeElem);
	}

	/**
	 * Fills an {@link ObjectLocationData.Hypothesis} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData.Hypothesis} object to fill with
	 *            all the information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, Path3DData type)
			throws ParsingException {

		Type.fromElement(dataElement, type);

		type.setPolygon(PrecisePolygon3D.fromElement(dataElement
				.getFirstChildElement(PrecisePolygon3D.getBaseTag())));
		type.setScope(Scope.valueOf(dataElement.getFirstChildElement(
				SCOPE_TAG_NAME).getValue()));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp()
				+ " scope: " + scope + " Polygon: " + polygon + "]";

	}
}
