package de.unibi.airobots.btl.data.medicalreport;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.ArrayList;

import de.unibi.airobots.btl.Type;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

/**
 * A collection of medicaldata.
 * 
 * @author: dnacke
 */
public class MedicalDataList extends Type {

	private ArrayList<MedicalData> medicalData;

	/** Constructor. */
	public MedicalDataList() {
		super();
		medicalData = new ArrayList<MedicalData>();
	}

	/** Adds new person data to this list. */
	public void addMedicalData(MedicalData data) {
		medicalData.add(data);
	}

	/** Returns the person data from the list. */
	public MedicalData getmedicalData(int index) {
		if (index > medicalData.size())
			return null;

		return medicalData.get(index);
	}

	/** Update an existing medicalData by index. */
	public void updateMedicalData(int index, MedicalData data) {
		medicalData.set(index, data);
	}

	/** Delete medicalData with given index. */
	public void deleteMedicalData(int index) {
		medicalData.remove(index);
	}

	/** Removes all person data from this list. */
	public void clearList() {
		medicalData.clear();
	}

	/** Returns the number of stored person data. */
	public int size() {
		return medicalData.size();
	}

	/** Returns the base tag as a string. */
	public static String getBaseTag() {
		return "PERSONS";
	}

	/** {@inheritDoc} */
	public static void fromElement(Element element, MedicalDataList type) {

		Type.fromElement(element, type);

		Nodes nodes = element.query(MedicalData.getBaseTag());
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element medicalElement = (Element) node;

				MedicalData parsed = new MedicalData();
				MedicalData.fromElement(medicalElement, parsed);
				type.addMedicalData(parsed);

			}

		}

	}

	/** {@inheritDoc} */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		int nrMedicalData = medicalData.size();
		for (int i = 0; i < nrMedicalData; i++) {

			Element medicalDataElement = new Element(MedicalData.getBaseTag());
			medicalData.get(i).fillInto(medicalDataElement);
			parent.appendChild(medicalDataElement);

		}

	}

}
