package de.unibi.airobots.btl.data.person;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.ArrayList;
import java.util.List;

import de.unibi.airobots.btl.Type;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

// ==========================================================================
// We should adapt the output of the boost detectors to the btl library. By
// now the classes for the found faces are adapted to the original output of
// the boost detector to make sure other components could continue work.
// ==========================================================================

/**
 * A list of found faces. This class is based on the original output of the
 * boost detector.
 * 
 * @author: dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class FaceList extends Type {

	/** A list of all faces in this face list. */
	private List<Face> faces;

	/** Constructor. */
	public FaceList() {
		faces = new ArrayList<Face>();
	}

	public void addFace(Face face) {
		faces.add(face);
	}

	public List<Face> getFaces() {
		return faces;
	}

	public static String getBaseTag() {
		return "OBJECTS";
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		parent.addAttribute(new Attribute("creator", getGenerator()));

		for (Face face : faces) {

			Element faceElement = new Element(Face.getBaseTag());
			face.fillInto(faceElement);

			parent.appendChild(faceElement);

		}

	}

	/** @{inheritDoc */
	public static void fromElement(Element element, FaceList list) {

		Nodes nodes = element.query(Face.getBaseTag());
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;

				Face face = new Face();
				Face.fromElement(elem, face);

				list.addFace(face);

			}

		}

	}

}
