/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * PlaneDataTest.cpp
 *
 *  Created on: Jul 21, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/geometry/BoundingBox3D.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(BoundingBox3DTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		vector<pair<double, double> > polygon;
		polygon.push_back(pair<double, double>(0, 0));
		polygon.push_back(pair<double, double>(1, 1));
		polygon.push_back(pair<double, double>(2, 2));

		BoundingBox3D original;

		original.setGenerator("generator123");
		original.setTimestamp(Timestamp());

		Rotation3D rotation(1.0, 0, 0, 90, DEGREE);
		original.setRotation(rotation);

		Point3D origin(1.0, 1.0, 1.0, MILLIMETER);
		original.setOrigin(origin);

		original.setWidth(2.0, MILLIMETER);
		original.setDepth(3.0, MILLIMETER);
		original.setHeight(4.0, MILLIMETER);

		Location originalLoc(BoundingBox3D::getTemplate(),
				BoundingBox3D::getBaseTag());

		originalLoc = original;

		BoundingBox3D parsed = extract<BoundingBox3D>(originalLoc);

		BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_CLOSE(original.getOrigin().getX(MILLIMETER),
				parsed.getOrigin().getX(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getOrigin().getY(MILLIMETER),
				parsed.getOrigin().getY(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getOrigin().getZ(MILLIMETER),
				parsed.getOrigin().getZ(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getWidth(MILLIMETER),
				original.getWidth(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getDepth(MILLIMETER),
				parsed.getDepth(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getHeight(MILLIMETER),
				parsed.getHeight(MILLIMETER), TOLERANCE);

		BOOST_REQUIRE(original.getRotation() == parsed.getRotation());

		BOOST_REQUIRE(original == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("BoundingBox3D.xml"),
			BoundingBox3D::getBaseTag());
	BoundingBox3D parsed = extract<BoundingBox3D>(fileContents);

	Eigen::Matrix3d mat = parsed.getRotation().getMatrix();

	BOOST_REQUIRE_EQUAL("TestApp", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(1.0, parsed.getOrigin().getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getOrigin().getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(3.0, parsed.getOrigin().getZ(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(4.0, parsed.getWidth(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(5.0, parsed.getDepth(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(6.0, parsed.getHeight(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(0, (double)mat(0,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1, (double)mat(0,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2, (double)mat(0,2), TOLERANCE);
	BOOST_REQUIRE_CLOSE(3, (double)mat(1,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(4, (double)mat(1,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(5, (double)mat(1,2), TOLERANCE);
	BOOST_REQUIRE_CLOSE(6, (double)mat(2,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(7, (double)mat(2,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(8, (double)mat(2,2), TOLERANCE);

}
BOOST_AUTO_TEST_SUITE_END()
