/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * KatanaPoseDataTest.cpp
 *
 *  Created on: May 25, 2010
 *      Author: ttoenige
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/grasp/KatanaPoseData.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(KatanaPoseDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try
	{

		KatanaPoseData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setX(222,MILLIMETER);
		original.setY(333,MILLIMETER);
		original.setZ(444,MILLIMETER);
		original.setPhi(1.11,RADIAN);
		original.setPsi(0.55,RADIAN);
		original.setTheta(1.23,RADIAN);

		Location originalLoc(KatanaPoseData::getTemplate(), KatanaPoseData::getBaseTag());
		originalLoc = original;

		KatanaPoseData parsed = extract<KatanaPoseData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getX(MILLIMETER), original.getX(MILLIMETER));
		BOOST_REQUIRE_EQUAL(parsed.getY(MILLIMETER), original.getY(MILLIMETER));
		BOOST_REQUIRE_EQUAL(parsed.getZ(MILLIMETER), original.getZ(MILLIMETER));
		BOOST_REQUIRE_EQUAL(parsed.getTheta(RADIAN), original.getTheta(RADIAN) );
		BOOST_REQUIRE_EQUAL(parsed.getPhi(RADIAN), original.getPhi(RADIAN));
		BOOST_REQUIRE_EQUAL(parsed.getPsi(RADIAN), original.getPsi(RADIAN));

	}
	catch (std::exception e)
	{
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility)
{
	Location fileContents(readFile("KatanaPoseData.xml"), KatanaPoseData::getBaseTag());
	KatanaPoseData parsed = extract<KatanaPoseData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(232, parsed.getX(MILLIMETER));
	BOOST_REQUIRE_EQUAL(343, parsed.getY(MILLIMETER));
	BOOST_REQUIRE_EQUAL(454, parsed.getZ(MILLIMETER));
	BOOST_REQUIRE_EQUAL(1.34, parsed.getPhi(RADIAN));
	BOOST_REQUIRE_EQUAL(1.23, parsed.getPsi(RADIAN));
	BOOST_REQUIRE_EQUAL(0.23, parsed.getTheta(RADIAN));
}

BOOST_AUTO_TEST_SUITE_END()
