/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#define _USE_MATH_DEFINES
#include <math.h>

#include "btl/data/hardwareinfo/LaserInfo.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;

BOOST_AUTO_TEST_SUITE(LaserInfoTest)

BOOST_AUTO_TEST_CASE(testGetScanNum) {

	BOOST_REQUIRE_EQUAL(0, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, -M_PI / 2.0));
	BOOST_REQUIRE_EQUAL(360, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, M_PI / 2.0));
	BOOST_REQUIRE_EQUAL(180, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, 0.0));

	BOOST_REQUIRE_EQUAL(0, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, -M_PI / 2.0 + 0.0001));
	BOOST_REQUIRE_EQUAL(360, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, M_PI / 2.0 - 0.0001));

	double nextAngle = -M_PI / 2.0 + (0.5 * M_PI / 180.0);
	BOOST_REQUIRE_EQUAL(1, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, nextAngle));
	BOOST_REQUIRE_EQUAL(1, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, nextAngle - 0.0001));

	double prevAngle = M_PI / 2.0 - (0.5 * M_PI / 180.0);
	BOOST_REQUIRE_EQUAL(359, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, prevAngle));
	BOOST_REQUIRE_EQUAL(359, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, prevAngle + 0.0001));

	try{
		LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, M_PI);
		BOOST_FAIL(string("OutOfRangeException has not been thrown although it should have been"));
	} catch (exception e){

	}

}

BOOST_AUTO_TEST_SUITE_END()
