/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * RegionDataTest.cpp
 *
 *  Created on: Jun 23, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/navigation/Path3DData.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const float TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(Path3DDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		vector<triple<float, float, float> > polygon;
		polygon.push_back(triple<float, float, float>(0,0,0));
		polygon.push_back(triple<float, float, float>(1,1,1));
		polygon.push_back(triple<float, float, float>(2,2,2));

		Path3DData original;
		original.setScope(Path3DData::LOCAL);
		original.setPolygon(polygon, MILLIMETER);

		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, original.getPolygon(CENTIMETER)[2].first * 10, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, original.getPolygon(METER)[2].first * 1000, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, original.getPolygon(MILLIMETER)[2].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].second, original.getPolygon(MILLIMETER)[2].second, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].third, original.getPolygon(MILLIMETER)[2].third, TOLERANCE);
		original.setPolygon(polygon, MILLIMETER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 2.0f, TOLERANCE);
		original.setPolygon(polygon, METER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 2000.0f, TOLERANCE);
		original.setPolygon(polygon, CENTIMETER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 20.0f, TOLERANCE);
		original.setPolygon(polygon, MILLIMETER);

		Location originalLoc(Path3DData::getTemplate(), Path3DData::getBaseTag());

		originalLoc = original;

		Path3DData parsed = extract<Path3DData>(originalLoc);

		BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_EQUAL(original.getScope(), parsed.getScope());
		BOOST_REQUIRE_EQUAL(original.getPolygon(MILLIMETER).size(), parsed.getPolygon(MILLIMETER).size());
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[0].first, parsed.getPolygon(MILLIMETER)[0].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[1].first, parsed.getPolygon(MILLIMETER)[1].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, parsed.getPolygon(MILLIMETER)[2].first, TOLERANCE);

		BOOST_REQUIRE(original == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Path3DData.xml"), Path3DData::getBaseTag());
	Path3DData parsed = extract<Path3DData>(fileContents);

	BOOST_REQUIRE_EQUAL("PlaneDetector", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(Path3DData::LOCAL, parsed.getScope());
	BOOST_REQUIRE_EQUAL(3.0f, parsed.getPolygon(MILLIMETER).size());
	BOOST_REQUIRE_CLOSE(0.0f, parsed.getPolygon(MILLIMETER)[0].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0f, parsed.getPolygon(MILLIMETER)[1].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0f, parsed.getPolygon(MILLIMETER)[2].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0f, parsed.getPolygon(MILLIMETER)[2].second, TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0f, parsed.getPolygon(MILLIMETER)[2].third, TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
