/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * ObjectDataTest.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/object/ObjectData.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ObjectDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		ObjectData original;

		ObjectData::Hypothesis originalHyp;
		originalHyp.setGenerator("test");
		originalHyp.setTimestamp(Timestamp(253543797,253543712));
		originalHyp.setReliability(0.1);
		originalHyp.setClassLabel("smacks");

		original.addHypothesis(originalHyp);

		Location originalLoc(ObjectData::getTemplate(), ObjectData::getBaseTag());
		originalLoc = original;

		ObjectData parsed = extract<ObjectData>(originalLoc);
		ObjectData::Hypothesis parsedHyp = parsed.getHypotheses()[0];

		BOOST_REQUIRE_EQUAL(parsedHyp.getGenerator(), originalHyp.getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp.getTimestamp(), originalHyp.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsedHyp.getReliability(), originalHyp.getReliability(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsedHyp.getClassLabel(), originalHyp.getClassLabel());

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("ObjectData.xml"), ObjectData::getBaseTag());
	ObjectData parsed = extract<ObjectData>(fileContents);
	ObjectData::Hypothesis parsedHyp = parsed.getHypotheses()[0];

	BOOST_REQUIRE_EQUAL("BoostDetector", parsedHyp.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsedHyp.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsedHyp.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.6, parsedHyp.getReliability(), 0.0001);
	BOOST_REQUIRE_EQUAL("smacks", parsedHyp.getClassLabel());

}

BOOST_AUTO_TEST_SUITE_END()
