/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/data/speechrec/Voice.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(VoiceTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		Voice original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setAngle(21312.78,RADIAN);
		original.setRuntimeDifference(-1.023,MILLISECONDS);
		original.setAngleVariance(0.45,RADIAN);
		original.setRuntimeDifferenceVariance(1.34,MILLISECONDS);

		Location originalLoc(Voice::getTemplate(), Voice::getBaseTag());
		originalLoc = original;

		Voice parsed = extract<Voice>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(RADIAN), original.getAngle(RADIAN), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getRuntimeDifference(MICROSECONDS), original.getRuntimeDifference(MICROSECONDS));
		BOOST_REQUIRE_CLOSE(parsed.getAngleVariance(RADIAN), original.getAngleVariance(RADIAN), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getRuntimeDifferenceVariance(MICROSECONDS), original.getRuntimeDifferenceVariance(MICROSECONDS));

		BOOST_REQUIRE_EQUAL(parsed, original);
		parsed.setGenerator("trash");
		BOOST_REQUIRE(!(parsed == original));
		parsed.setGenerator(original.getGenerator());
		parsed.setRuntimeDifference(999,MICROSECONDS);
		BOOST_REQUIRE(!(parsed == original));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Voice.xml"), Voice::getBaseTag());
	Voice parsed = extract<Voice>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(RADIAN), tolerance);
	BOOST_REQUIRE_EQUAL(77230, parsed.getRuntimeDifference(MICROSECONDS));
	BOOST_REQUIRE_CLOSE((float) 0.345, parsed.getAngleVariance(RADIAN), tolerance);
	BOOST_REQUIRE_EQUAL(1340, parsed.getRuntimeDifferenceVariance(MICROSECONDS));

}

BOOST_AUTO_TEST_SUITE_END()
