package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;

import javax.vecmath.Matrix3d;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.geometry.Cylinder;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.data.geometry.Rotation3D;
import de.unibi.airobots.btl.units.*;


/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class CylinderTest {

	@Test
	public void selfCompatibility() throws Exception {

		double[] m = { 3.0, 45.0, 2.0, 0.5, 0.1, 2.9, 3.89, 0.4, 23.9 };
		Matrix3d mat = new Matrix3d(m);

		Rotation3D r = new Rotation3D(mat);
		Point3D p = new Point3D(1, 2, 3, LengthUnit.MILLIMETER);

		Cylinder c0 = new Cylinder(p, r, 21.0, 42.0, LengthUnit.MILLIMETER);
		Cylinder c1 = new Cylinder(p, r, 21.0, 42.0, LengthUnit.MILLIMETER);

		assertTrue("equals() not working", c0.equals(c1));
		assertTrue("equals() not working", c1.equals(c0));

		c1.setHeight(45, LengthUnit.METER);

		assertFalse("equals() not working", c0.equals(c1));

		Element element = new Element(Cylinder.getBaseTag());
		c0.fillInto(element);
		
		System.out.println(element.toXML());

		Cylinder c2 = new Cylinder();
		Cylinder.fromElement(element, c2);

		assertEquals("serializing not working", c0, c2);
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("Cylinder.xml"));

		Cylinder parsed = new Cylinder();
		Cylinder.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), Cylinder.getBaseTag());

		assertEquals(0.0, parsed.getPosition().getX(LengthUnit.MILLIMETER),
				0.00001);
		assertEquals(1.0, parsed.getPosition().getY(LengthUnit.MILLIMETER),
				0.00001);
		assertEquals(2.0, parsed.getPosition().getZ(LengthUnit.MILLIMETER),
				0.00001);
		assertEquals(6.0, parsed.getHeight(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(7.0, parsed.getRadius(LengthUnit.MILLIMETER), 0.00001);

		assertEquals(0.0, parsed.getOrientation().getMatrix().m00, 0.00001);
		assertEquals(1.0, parsed.getOrientation().getMatrix().m01, 0.00001);
		assertEquals(2.0, parsed.getOrientation().getMatrix().m02, 0.00001);
		assertEquals(3.0, parsed.getOrientation().getMatrix().m10, 0.00001);
		assertEquals(4.0, parsed.getOrientation().getMatrix().m11, 0.00001);
		assertEquals(5.0, parsed.getOrientation().getMatrix().m12, 0.00001);
		assertEquals(6.0, parsed.getOrientation().getMatrix().m20, 0.00001);
		assertEquals(7.0, parsed.getOrientation().getMatrix().m21, 0.00001);
		assertEquals(8.0, parsed.getOrientation().getMatrix().m22, 0.00001);
	}
}
