package de.unibi.airobots.btl.data.navigation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Document;
import nu.xom.Element;
import org.apache.log4j.Logger;
import org.junit.Test;


/**
 * Test cases for {@link ROSBasePlannerConfigTest}
 * 
 * @author pdressel
 */
public class ROSBaseLocalPlannerConfigTest {
    
    private final static Double DELTA = 0.0001;
    private Logger logger = Logger.getLogger(getClass());
        
    public static ROSBaseLocalPlannerConfiguration createMockObject() {
        
        ROSBaseLocalPlannerConfiguration baseLocalPlannerConfig = new ROSBaseLocalPlannerConfiguration();
        baseLocalPlannerConfig.setAccLimTheta(Math.random());
        baseLocalPlannerConfig.setAccLimX(Math.random());
        baseLocalPlannerConfig.setAccLimY(Math.random());
        baseLocalPlannerConfig.setAngularSimGranularity(Math.random());
        baseLocalPlannerConfig.setDwa(Math.random() > Math.random());
        baseLocalPlannerConfig.setEscapeResetDist((int)Math.random());
        baseLocalPlannerConfig.setEscapeResetTheta(Math.random());
        baseLocalPlannerConfig.setEscapeVel(Math.random());
        baseLocalPlannerConfig.setGdistScale(Math.random());
        baseLocalPlannerConfig.setHeadingLookahead(Math.random());
        baseLocalPlannerConfig.setHeadingScoring(Math.random() > Math.random());
        baseLocalPlannerConfig.setHeadingScoringTimestep(Math.random());
        baseLocalPlannerConfig.setHolonomicRobot(Math.random() > Math.random());
        baseLocalPlannerConfig.setMaxVelTheta(Math.random());
        baseLocalPlannerConfig.setMaxVelX(Math.random());
        baseLocalPlannerConfig.setMinInPlaceVelTheta(Math.random());
        baseLocalPlannerConfig.setMinVelTheta(Math.random());
        baseLocalPlannerConfig.setMinVelX(Math.random());
        baseLocalPlannerConfig.setOccdistScale(Math.random());
        baseLocalPlannerConfig.setOscillationResetDist(Math.random());
        baseLocalPlannerConfig.setPdistScale(Math.random());
        baseLocalPlannerConfig.setRestoreDefaults(Math.random() > Math.random());
        baseLocalPlannerConfig.setSimGranularity(Math.random());
        baseLocalPlannerConfig.setSimTime(Math.random());
        baseLocalPlannerConfig.setSimpleAttractor(Math.random() > Math.random());
        baseLocalPlannerConfig.setVthetaSamples((int)Math.random());
        baseLocalPlannerConfig.setVxSamples((int)Math.random());
        baseLocalPlannerConfig.setYVels("[-0.3, -0.1, 0.1, 0.3]");
        
        return baseLocalPlannerConfig;
    }
    
    
    @Test
    public void selfCompatibility() throws Exception {
        
        ROSBaseLocalPlannerConfiguration original = createMockObject();
        ROSBaseLocalPlannerConfiguration parsed = new ROSBaseLocalPlannerConfiguration();
        
        Document doc = new Document(new Element(ROSBaseLocalPlannerConfiguration.getBaseTag()));
        original.fillInto(doc.getRootElement());
        logger.info(doc.toXML());
        
        ROSBaseLocalPlannerConfiguration.fromElement(doc.getRootElement(), parsed);
        
        assertEquals(original.getAccLimTheta(), parsed.getAccLimTheta(), DELTA);
        assertEquals(original.getAccLimX(), parsed.getAccLimX(), DELTA);
        assertEquals(original.getAccLimY(), parsed.getAccLimY(), DELTA);
        assertEquals(original.getAngularSimGranularity(), parsed.getAngularSimGranularity(), DELTA);
        assertEquals(original.isDwa(), parsed.isDwa());
        assertEquals(original.getEscapeResetDist(), parsed.getEscapeResetDist(), DELTA);
        assertEquals(original.getEscapeResetTheta(), parsed.getEscapeResetTheta(), DELTA);
        assertEquals(original.getEscapeVel(), parsed.getEscapeVel(), DELTA);
        assertEquals(original.getGdistScale(), parsed.getGdistScale(), DELTA);
        assertEquals(original.getHeadingLookahead(), parsed.getHeadingLookahead(), DELTA);
        assertEquals(original.isHeadingScoring(), parsed.isHeadingScoring());
        assertEquals(original.getHeadingScoringTimestep(), parsed.getHeadingScoringTimestep(), DELTA);
        assertEquals(original.isHolonomicRobot(), parsed.isHolonomicRobot());
        assertEquals(original.getMaxVelTheta(), parsed.getMaxVelTheta(), DELTA);
        assertEquals(original.getMaxVelX(), parsed.getMaxVelX(), DELTA);
        assertEquals(original.getMinInPlaceVelTheta(), parsed.getMinInPlaceVelTheta(), DELTA);
        assertEquals(original.getMinVelTheta(), parsed.getMinVelTheta(), DELTA);
        assertEquals(original.getMaxVelX(), parsed.getMaxVelX(), DELTA);
        assertEquals(original.getMinInPlaceVelTheta(), parsed.getMinInPlaceVelTheta(), DELTA);
        assertEquals(original.getMinVelTheta(), parsed.getMinVelTheta(), DELTA);
        assertEquals(original.getOccdistScale(), parsed.getOccdistScale(), DELTA);
        assertEquals(original.getOscillationResetDist(), parsed.getOscillationResetDist(), DELTA);
        assertEquals(original.getPdistScale(), parsed.getPdistScale(), DELTA);
        assertEquals(original.isRestoreDefaults(), parsed.isRestoreDefaults());
        assertEquals(original.getSimGranularity(), parsed.getSimGranularity(), DELTA);
        assertEquals(original.getSimTime(), parsed.getSimTime(), DELTA);
        assertEquals(original.isSimpleAttractor(), parsed.isSimpleAttractor());
        assertEquals(original.getVthetaSamples(), parsed.getVthetaSamples());
        assertEquals(original.getVxSamples(), parsed.getVxSamples());
        assertEquals(original.getYVels(), parsed.getYVels());
    }    
}
