package de.unibi.airobots.btl.data.object;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.data.geometry.Polygon;
import de.unibi.airobots.btl.data.object.ObjectData;
import de.unibi.airobots.btl.data.object.ObjectLocationData;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class ObjectShapeDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		final long timestamp = System.currentTimeMillis();

		Polygon poly = new Polygon();
		poly.addPoint(0, 0);
		poly.addPoint(1, 1);
		poly.addPoint(2, 2);

		ObjectShapeData original = new ObjectShapeData();
		original.setPolygon(poly);
		original.setSensorType("SONY_EVI");
		original.setCenter(new Point3D(1, 2, 4, LengthUnit.MILLIMETER));
		original.setHeight(34, LengthUnit.MILLIMETER);
		original.setWidth(32, LengthUnit.METER);
		original.setDepth(38, LengthUnit.MILLIMETER);
		
		original = new ObjectShapeData(original);

		ObjectShapeData.Hypothesis originalHyp = new ObjectShapeData.Hypothesis();

		originalHyp.setTimestamp(timestamp);
		originalHyp.setGenerator("test");
		originalHyp.setReliability(0.1);

		originalHyp.setClassLabel("testclass");

		original.addHypothesis(originalHyp);

		Document doc = new Document(new Element(ObjectData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		ObjectShapeData parsed = new ObjectShapeData();
		ObjectShapeData.fromElement(doc.getRootElement(), parsed);
		ObjectShapeData.Hypothesis parsedHyp = parsed.getShapeHypotheses()
				.iterator().next();

		assertEquals(originalHyp.getGenerator(), parsedHyp.getGenerator());
		assertEquals(originalHyp.getTimestamp(), parsedHyp.getTimestamp());
		assertEquals(originalHyp.getClassLabel(), parsedHyp.getClassLabel());
		assertEquals(originalHyp.getReliability(), parsedHyp.getReliability(),
				0.0001);
		assertEquals(original.getSensorType(), parsed.getSensorType());
		assertEquals(original.getPolygon(), parsed.getPolygon());
		assertEquals(original.getCenter(), parsed.getCenter());
		assertEquals(original.getDepth(LengthUnit.MILLIMETER),
				parsed.getDepth(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getHeight(LengthUnit.MILLIMETER),
				parsed.getHeight(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getWidth(LengthUnit.MILLIMETER),
				parsed.getWidth(LengthUnit.MILLIMETER), 0.0001);

		System.out.println(parsed);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Polygon poly = new Polygon();
		poly.addPoint(0, 0);
		poly.addPoint(1, 1);
		poly.addPoint(2, 2);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("ObjectShapeData.xml"));

		ObjectShapeData parsed = new ObjectShapeData();
		ObjectShapeData.fromElement(doc.getRootElement(), parsed);
		ObjectShapeData.Hypothesis parsedHyp = parsed.getShapeHypotheses()
				.iterator().next();
		Point3D center = parsed.getCenter();

		assertEquals(doc.getRootElement().getLocalName(),
				ObjectShapeData.getBaseTag());

		assertEquals("BoostDetector", parsedHyp.getGenerator());
		assertEquals(new Timestamp(253543797l, 253543712l),
				parsedHyp.getTimestamp());
		assertEquals("smacks", parsedHyp.getClassLabel());
		assertEquals(0.6, parsedHyp.getReliability(), 0.002);
		assertEquals("SONY_EVI", parsed.getSensorType());
		assertEquals(poly, parsed.getPolygon());
		assertEquals(1.0, parsed.getHeight(LengthUnit.MILLIMETER), 0.002);
		assertEquals(3.0, parsed.getDepth(LengthUnit.MILLIMETER), 0.002);
		assertEquals(4.0, parsed.getWidth(LengthUnit.MILLIMETER), 0.002);

	}

}
