package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.assertEquals;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.TypeTest;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.speechrec.GPSRAction.GPSRParameter;

/**
 * Test cases for {@link Type}.
 * 
 * @author sjebbara
 */
public class GrammarNonTerminalTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		GrammarNonTerminal actual = new GrammarNonTerminal();
		final long timestamp = System.currentTimeMillis();
		actual.setTimestamp(timestamp);
		actual.setGenerator("test");
		actual.setName("Phrase");

		UtterancePart utt = new UtterancePart();
		final long uttTimestamp = System.currentTimeMillis();
		utt.setTimestamp(uttTimestamp);
		utt.setGenerator("test");
		utt.setWord("das");
		utt.setBegin(1);
		utt.setId(2);
		utt.setEnd(3);
		utt.setAcousticScore(4.567);
		utt.setLmScore(5.678);
		utt.setCombinedScore(6.789);

		GrammarNonTerminal nt = new GrammarNonTerminal();
		final long ntTimestamp = System.currentTimeMillis();
		nt.setTimestamp(ntTimestamp);
		nt.setGenerator("test");
		nt.setName("Nomen");

		actual.addSymbol(utt);
		actual.addSymbol(nt);

		Document doc = new Document(
				new Element(GrammarNonTerminal.getBaseTag()));
		actual.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		GrammarNonTerminal parsed = new GrammarNonTerminal();
		GrammarNonTerminal.fromElement(doc.getRootElement(), parsed);

		assertEquals(actual, parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {

		GrammarNonTerminal actual = new GrammarNonTerminal();
		final long timestamp = 12345;
		actual.setTimestamp(timestamp);
		actual.setGenerator("test");
		actual.setName("Phrase");

		UtterancePart utt = new UtterancePart();
		final long uttTimestamp = 12345;
		utt.setTimestamp(uttTimestamp);
		utt.setGenerator("test");
		utt.setWord("das");
		utt.setBegin(1);
		utt.setId(2);
		utt.setEnd(3);
		utt.setAcousticScore(4.567);
		utt.setLmScore(5.678);
		utt.setCombinedScore(6.789);

		GrammarNonTerminal nt = new GrammarNonTerminal();
		final long ntTimestamp = 12345;
		nt.setTimestamp(ntTimestamp);
		nt.setGenerator("test");
		nt.setName("Nomen");

		actual.addSymbol(utt);
		actual.addSymbol(nt);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("GrammarNonTerminal.xml"));

		GrammarNonTerminal parsed = new GrammarNonTerminal();
		GrammarNonTerminal.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(),
				GrammarNonTerminal.getBaseTag());
		System.out.println(actual.getSubsymbols());
		System.out.println(parsed.getSubsymbols());

		assertEquals(actual, parsed);
	}
}
